@@ -7,8 +7,9 @@ import pytest
 
 from xonsh.procs.posix import PopenThread
 from xonsh.procs.proxies import STDOUT_DISPATCHER, ProcProxy, ProcProxyThread
-from xonsh.procs.specs import cmds_to_specs, run_subproc
+from xonsh.procs.specs import SubprocSpec, cmds_to_specs, run_subproc
 from xonsh.pytest.tools import skip_if_on_windows
+from xonsh.tools import XonshError
 
 
 @skip_if_on_windows
@@ -138,7 +139,7 @@ def test_capture_always(
     [
         ("object", False),
         ("stdout", True),
-        ("hiddenobject", True),
+        ("hiddenobject", False),
         (False, True),
     ],
 )
@@ -166,3 +167,62 @@ def test_callable_alias_cls(thread_subprocs, xession):
     spec = cmds_to_specs(cmds, captured="stdout")[0]
     proc = spec.run()
     assert proc.f == obj
+
+
+@pytest.mark.parametrize("captured", ["hiddenobject", False])
+def test_procproxy_not_captured(xession, captured):
+    xession.aliases["tst"] = lambda: 0
+    cmds = (["tst", "/root"],)
+
+    xession.env["THREAD_SUBPROCS"] = False
+    specs = cmds_to_specs(cmds, captured)
+
+    assert specs[0].cls is ProcProxy
+
+    # neither stdout nor stderr should be captured
+    assert specs[0].stdout is None
+    assert specs[0].stderr is None
+
+
+def test_on_command_not_found_fires(xession):
+    xession.env.update(
+        dict(
+            XONSH_INTERACTIVE=True,
+        )
+    )
+
+    fired = False
+
+    def my_handler(cmd, **kwargs):
+        nonlocal fired
+        assert cmd[0] == "xonshcommandnotfound"
+        fired = True
+
+    xession.builtins.events.on_command_not_found(my_handler)
+    subproc = SubprocSpec.build(["xonshcommandnotfound"])
+    with pytest.raises(XonshError) as expected:
+        subproc.run()
+    assert "command not found: xonshcommandnotfound" in str(expected.value)
+    assert fired
+
+
+def test_on_command_not_found_doesnt_fire_in_non_interactive_mode(xession):
+    xession.env.update(
+        dict(
+            XONSH_INTERACTIVE=False,
+        )
+    )
+
+    fired = False
+
+    def my_handler(cmd, **kwargs):
+        nonlocal fired
+        assert cmd[0] == "xonshcommandnotfound"
+        fired = True
+
+    xession.builtins.events.on_command_not_found(my_handler)
+    subproc = SubprocSpec.build(["xonshcommandnotfound"])
+    with pytest.raises(XonshError) as expected:
+        subproc.run()
+    assert "command not found: xonshcommandnotfound" in str(expected.value)
+    assert not fired
