@@ -127,6 +127,38 @@ COMMAND_EXAMPLES = (
             suffix="b",
         ),
     ),
+    (
+        f"command >/dev/nul{X}",
+        CommandContext(
+            args=(CommandArg("command"), CommandArg(">", is_io_redir=True)),
+            arg_index=2,
+            prefix="/dev/nul",
+        ),
+    ),
+    (
+        f"command 2>/dev/nul{X}",
+        CommandContext(
+            args=(CommandArg("command"), CommandArg("2>", is_io_redir=True)),
+            arg_index=2,
+            prefix="/dev/nul",
+        ),
+    ),
+    (
+        f"command >>/dev/nul{X}",
+        CommandContext(
+            args=(CommandArg("command"), CommandArg(">>", is_io_redir=True)),
+            arg_index=2,
+            prefix="/dev/nul",
+        ),
+    ),
+    (
+        f"command </dev/nul{X}",
+        CommandContext(
+            args=(CommandArg("command"), CommandArg("<", is_io_redir=True)),
+            arg_index=2,
+            prefix="/dev/nul",
+        ),
+    ),
 )
 
 EMPTY_COMMAND_EXAMPLES = (
