@@ -1,15 +1,11 @@
-# -*- coding: utf-8 -*-
 """Testing dirstack"""
-from __future__ import unicode_literals, print_function
 
-from contextlib import contextmanager
 import os
+from contextlib import contextmanager
 
 import pytest  # noqa F401
 
 from xonsh import dirstack
-from xonsh.environ import Env
-
 
 HERE = os.path.abspath(os.path.dirname(__file__))
 PARENT = os.path.dirname(HERE)
@@ -24,7 +20,7 @@ def chdir(adir):
 
 
 def test_simple(xession):
-    xession.env = Env(CDPATH=PARENT, PWD=PARENT)
+    xession.env.update(dict(CDPATH=PARENT, PWD=PARENT))
     with chdir(PARENT):
         assert os.getcwd() != HERE
         dirstack.cd(["tests"])
@@ -32,7 +28,7 @@ def test_simple(xession):
 
 
 def test_cdpath_simple(xession):
-    xession.env = Env(CDPATH=PARENT, PWD=HERE)
+    xession.env.update(dict(CDPATH=PARENT, PWD=HERE))
     with chdir(os.path.normpath("/")):
         assert os.getcwd() != HERE
         dirstack.cd(["tests"])
@@ -40,7 +36,7 @@ def test_cdpath_simple(xession):
 
 
 def test_cdpath_collision(xession):
-    xession.env = Env(CDPATH=PARENT, PWD=HERE)
+    xession.env.update(dict(CDPATH=PARENT, PWD=HERE))
     sub_tests = os.path.join(HERE, "tests")
     if not os.path.exists(sub_tests):
         os.mkdir(sub_tests)
@@ -51,7 +47,7 @@ def test_cdpath_collision(xession):
 
 
 def test_cdpath_expansion(xession):
-    xession.env = Env(HERE=HERE, CDPATH=("~", "$HERE"))
+    xession.env.update(dict(HERE=HERE, CDPATH=("~", "$HERE")))
     test_dirs = (
         os.path.join(HERE, "xonsh-test-cdpath-here"),
         os.path.expanduser("~/xonsh-test-cdpath-home"),
@@ -62,7 +58,7 @@ def test_cdpath_expansion(xession):
                 os.mkdir(d)
             assert os.path.exists(
                 dirstack._try_cdpath(d)
-            ), "dirstack._try_cdpath: could not resolve {0}".format(d)
+            ), f"dirstack._try_cdpath: could not resolve {d}"
     finally:
         for d in test_dirs:
             if os.path.exists(d):
@@ -70,7 +66,7 @@ def test_cdpath_expansion(xession):
 
 
 def test_cdpath_events(xession, tmpdir):
-    xession.env = Env(CDPATH=PARENT, PWD=os.getcwd())
+    xession.env.update(dict(CDPATH=PARENT, PWD=os.getcwd()))
     target = str(tmpdir)
 
     ev = None
@@ -93,7 +89,7 @@ def test_cdpath_events(xession, tmpdir):
 
 
 def test_cd_autopush(xession, tmpdir):
-    xession.env = Env(CDPATH=PARENT, PWD=os.getcwd(), AUTO_PUSHD=True)
+    xession.env.update(dict(CDPATH=PARENT, PWD=os.getcwd(), AUTO_PUSHD=True))
     target = str(tmpdir)
 
     old_dir = os.getcwd()
