@@ -1,25 +1,21 @@
-# -*- coding: utf-8 -*-
 """Testing dirstack"""
-# from __future__ import unicode_literals, print_function
-
 import os
 import os.path
 import subprocess
 import sys
 
 import pytest
+
 from xonsh import dirstack
-from xonsh.environ import Env
-from xonsh.dirstack import DIRSTACK
+from xonsh.dirstack import DIRSTACK, _unc_tempDrives
 from xonsh.platform import ON_WINDOWS
-from xonsh.dirstack import _unc_tempDrives
 
 HERE = os.path.abspath(os.path.dirname(__file__))
 PARENT = os.path.dirname(HERE)
 
 
 def drive_in_use(letter):
-    return ON_WINDOWS and os.system("vol {}: 2>nul>nul".format(letter)) == 0
+    return ON_WINDOWS and os.system(f"vol {letter}: 2>nul>nul") == 0
 
 
 MAX_TEMP_DRIVES = 4
@@ -94,7 +90,7 @@ def shares_setup(tmpdir_factory):
 
 def test_pushdpopd(xession):
     """Simple non-UNC push/pop to verify we didn't break nonUNC case."""
-    xession.env = Env(CDPATH=PARENT, PWD=HERE)
+    xession.env.update(dict(CDPATH=PARENT, PWD=HERE))
 
     dirstack.cd([PARENT])
     owd = os.getcwd()
@@ -107,7 +103,7 @@ def test_pushdpopd(xession):
 
 
 def test_cd_dot(xession):
-    xession.env = Env(PWD=os.getcwd())
+    xession.env.update(dict(PWD=os.getcwd()))
 
     owd = os.getcwd().casefold()
     dirstack.cd(["."])
@@ -118,7 +114,7 @@ def test_cd_dot(xession):
 def test_uncpushd_simple_push_pop(xession, shares_setup):
     if shares_setup is None:
         return
-    xession.env = Env(CDPATH=PARENT, PWD=HERE)
+    xession.env.update(dict(CDPATH=PARENT, PWD=HERE))
     dirstack.cd([PARENT])
     owd = os.getcwd()
     assert owd.casefold() == xession.env["PWD"].casefold()
@@ -135,7 +131,7 @@ def test_uncpushd_simple_push_pop(xession, shares_setup):
 def test_uncpushd_push_to_same_share(xession, shares_setup):
     if shares_setup is None:
         return
-    xession.env = Env(CDPATH=PARENT, PWD=HERE)
+    xession.env.update(dict(CDPATH=PARENT, PWD=HERE))
 
     dirstack.cd([PARENT])
     owd = os.getcwd()
@@ -169,7 +165,7 @@ def test_uncpushd_push_other_push_same(xession, shares_setup):
     Then push to a again. Pop (check b unmapped and a still mapped), pop, pop (check a is unmapped)"""
     if shares_setup is None:
         return
-    xession.env = Env(CDPATH=PARENT, PWD=HERE)
+    xession.env.update(dict(CDPATH=PARENT, PWD=HERE))
 
     dirstack.cd([PARENT])
     owd = os.getcwd()
@@ -218,69 +214,56 @@ def test_uncpushd_push_base_push_rempath(xession):
     pass
 
 
-# really?  Need to cut-and-paste 2 flavors of this? fixture requires yield in defined function body, not callee
-@pytest.fixture()
-def with_unc_check_enabled():
-    if not ON_WINDOWS:
-        return
+@pytest.fixture
+def toggle_unc_check():
+    old_wval = None
 
-    import winreg
+    def _update_key(key_type, value: int):
+        import winreg
 
-    old_wval = 0
-    key = winreg.OpenKey(
-        winreg.HKEY_CURRENT_USER,
-        r"software\microsoft\command processor",
-        access=winreg.KEY_WRITE,
-    )
-    try:
-        wval, wtype = winreg.QueryValueEx(key, "DisableUNCCheck")
-        old_wval = wval  # if values was defined at all
-    except OSError:
-        pass
-    winreg.SetValueEx(key, "DisableUNCCheck", None, winreg.REG_DWORD, 0)
-    winreg.CloseKey(key)
+        old_wval = 0
+        with dirstack._win_reg_key(
+            key_type,
+            r"software\microsoft\command processor",
+            access=winreg.KEY_WRITE,
+        ) as key:
+            try:
+                wval, wtype = winreg.QueryValueEx(key, "DisableUNCCheck")
+                old_wval = wval  # if values was defined at all
+            except OSError:
+                pass
+            winreg.SetValueEx(key, "DisableUNCCheck", None, winreg.REG_DWORD, value)
+        return old_wval
 
-    yield old_wval
+    def update(value: int):
+        import winreg
 
-    key = winreg.OpenKey(
-        winreg.HKEY_CURRENT_USER,
-        r"software\microsoft\command processor",
-        access=winreg.KEY_WRITE,
-    )
-    winreg.SetValueEx(key, "DisableUNCCheck", None, winreg.REG_DWORD, old_wval)
-    winreg.CloseKey(key)
+        for key_type in (winreg.HKEY_CURRENT_USER, winreg.HKEY_LOCAL_MACHINE):
+            try:
+                return _update_key(key_type, value)
+            except OSError:
+                pass
+
+    def wrapper(value: int):
+        nonlocal old_wval
+        old_wval = update(value) or 0
+        return old_wval
+
+    yield wrapper
+
+    # revert back to old value
+    if old_wval is not None:
+        update(old_wval)
 
 
 @pytest.fixture()
-def with_unc_check_disabled():  # just like the above, but value is 1 to *disable* unc check
-    if not ON_WINDOWS:
-        return
+def with_unc_check_enabled(toggle_unc_check):
+    return toggle_unc_check(1)
 
-    import winreg
 
-    old_wval = 0
-    key = winreg.OpenKey(
-        winreg.HKEY_CURRENT_USER,
-        r"software\microsoft\command processor",
-        access=winreg.KEY_WRITE,
-    )
-    try:
-        wval, wtype = winreg.QueryValueEx(key, "DisableUNCCheck")
-        old_wval = wval  # if values was defined at all
-    except OSError:
-        pass
-    winreg.SetValueEx(key, "DisableUNCCheck", None, winreg.REG_DWORD, 1)
-    winreg.CloseKey(key)
-
-    yield old_wval
-
-    key = winreg.OpenKey(
-        winreg.HKEY_CURRENT_USER,
-        r"software\microsoft\command processor",
-        access=winreg.KEY_WRITE,
-    )
-    winreg.SetValueEx(key, "DisableUNCCheck", None, winreg.REG_DWORD, old_wval)
-    winreg.CloseKey(key)
+@pytest.fixture()
+def with_unc_check_disabled(toggle_unc_check):
+    return toggle_unc_check(0)
 
 
 @pytest.fixture()
@@ -292,7 +275,6 @@ def xonsh_builtins_cd(xession):
 
 
 @pytest.mark.skipif(not ON_WINDOWS, reason="Windows-only UNC functionality")
-@xfail_py310
 def test_uncpushd_cd_unc_auto_pushd(xonsh_builtins_cd, with_unc_check_enabled):
     xonsh_builtins_cd.env["AUTO_PUSHD"] = True
     so, se, rc = dirstack.cd([r"\\localhost\uncpushd_test_PARENT"])
@@ -304,7 +286,6 @@ def test_uncpushd_cd_unc_auto_pushd(xonsh_builtins_cd, with_unc_check_enabled):
 
 
 @pytest.mark.skipif(not ON_WINDOWS, reason="Windows-only UNC functionality")
-@xfail_py310
 def test_uncpushd_cd_unc_nocheck(xonsh_builtins_cd, with_unc_check_disabled):
     if with_unc_check_disabled == 0:
         return
@@ -313,7 +294,6 @@ def test_uncpushd_cd_unc_nocheck(xonsh_builtins_cd, with_unc_check_disabled):
 
 
 @pytest.mark.skipif(not ON_WINDOWS, reason="Windows-only UNC functionality")
-@xfail_py310
 def test_uncpushd_cd_unc_no_auto_pushd(xonsh_builtins_cd, with_unc_check_enabled):
     if with_unc_check_enabled == 0:
         return
