@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """Tests the json history backend."""
 # pylint: disable=protected-access
 
@@ -8,12 +7,12 @@ import pytest
 
 from xonsh.history.json import (
     JsonHistory,
+    _xhj_gc_bytes_to_rmfiles,
     _xhj_gc_commands_to_rmfiles,
     _xhj_gc_files_to_rmfiles,
     _xhj_gc_seconds_to_rmfiles,
-    _xhj_gc_bytes_to_rmfiles,
 )
-from xonsh.history.main import history_main, HistoryAlias
+from xonsh.history.main import HistoryAlias, history_main
 from xonsh.lazyjson import LazyJSON
 
 CMDS = ["ls", "cat hello kitty", "abc", "def", "touch me", "grep from me"]
@@ -170,10 +169,7 @@ def test_show_cmd_numerate(inp, commands, offset, hist, xession, capsys):
     for ts, cmd in enumerate(CMDS):  # populate the shell history
         hist.append({"inp": cmd, "rtn": 0, "ts": (ts + 1, ts + 1.5)})
 
-    exp = (
-        "{}: {}".format(base_idx + idx * step, cmd)
-        for idx, cmd in enumerate(list(commands))
-    )
+    exp = (f"{base_idx + idx * step}: {cmd}" for idx, cmd in enumerate(list(commands)))
     exp = "\n".join(exp)
 
     history_main(["show", "-n"] + shlex.split(inp))
@@ -196,7 +192,7 @@ def test_history_diff(tmpdir, xession, monkeypatch, capsys):
             # make sure that flush is complete
             time.sleep(0.1)
 
-    left, right = [str(f) for f in files]
+    left, right = (str(f) for f in files)
     history_main(["diff", left, right])
     out, err = capsys.readouterr()
     # make sure it is called
@@ -376,9 +372,8 @@ def test_history_getitem(index, exp, hist, xession):
         assert (entry.cmd, entry.out, entry.rtn, entry.ts) == exp
 
 
-import time
 import calendar
-
+import time
 
 HF_FIRST_DAY = calendar.timegm(time.struct_time((2018, 5, 13, 0, 0, 0, 0, 0, 0)))
 
