@@ -3,26 +3,24 @@ This requires Xonsh installed in venv or otherwise available on PATH
 """
 import os
 import shutil
-import tempfile
 import subprocess as sp
+import tempfile
 from pathlib import Path
 
 import pytest
 
 import xonsh
 from xonsh.dirstack import with_pushd
-
-from tools import (
+from xonsh.pytest.tools import (
+    ON_DARWIN,
+    ON_TRAVIS,
+    ON_WINDOWS,
+    skip_if_on_darwin,
     skip_if_on_msys,
     skip_if_on_unix,
     skip_if_on_windows,
-    skip_if_on_darwin,
-    ON_WINDOWS,
-    ON_DARWIN,
-    ON_TRAVIS,
 )
 
-
 PATH = (
     os.path.join(os.path.abspath(os.path.dirname(__file__)), "bin")
     + os.pathsep
@@ -58,6 +56,7 @@ def run_xonsh(
     env["XONSH_DEBUG"] = "0"  # was "1"
     env["XONSH_SHOW_TRACEBACK"] = "1"
     env["RAISE_SUBPROC_ERROR"] = "0"
+    env["FOREIGN_ALIASES_SUPPRESS_SKIP_MESSAGE"] = "1"
     env["PROMPT"] = ""
     xonsh = shutil.which("xonsh", path=PATH)
     args = [xonsh, "--no-rc"]
@@ -533,6 +532,114 @@ a | a | b | b
         "1" * 2 * 4,
         0,
     ),
+    # test $SHLVL
+    (
+        """
+# test parsing of $SHLVL
+
+$SHLVL = "1"
+echo $SHLVL # == 1
+
+$SHLVL = 1
+echo $SHLVL # == 1
+
+$SHLVL = "-13"
+echo $SHLVL # == 0
+
+$SHLVL = "error"
+echo $SHLVL # == 0
+
+$SHLVL = 999
+echo $SHLVL # == 999
+
+$SHLVL = 1000
+echo $SHLVL # == 1
+
+# sourcing a script should maintain $SHLVL
+
+$SHLVL = 5
+touch temp_shlvl_test.sh
+source-bash temp_shlvl_test.sh
+rm temp_shlvl_test.sh
+echo $SHLVL # == 5
+
+# creating a subshell should increment the child's $SHLVL and maintain the parents $SHLVL
+
+$SHLVL = 5
+xonsh -c r'echo $SHLVL' # == 6
+echo $SHLVL # == 5
+
+# replacing the current process with another process should derease $SHLVL
+# (so that if the new process is a shell, $SHLVL is maintained)
+
+$SHLVL = 5
+xexec python3 -c 'import os; print(os.environ["SHLVL"])' # == 4
+""",
+        """1
+1
+0
+0
+999
+1
+5
+6
+5
+4
+""",
+        0,
+    ),
+    # test $() inside piped callable alias
+    (
+        r"""
+def _callme(args):
+    result = $(python -c 'print("tree");print("car")')
+    print(result[::-1])
+    print('one\ntwo\nthree')
+
+aliases['callme'] = _callme
+callme | grep t
+""",
+        """eert
+two
+three
+""",
+        0,
+    ),
+    # test ![] inside piped callable alias
+    (
+        r"""
+def _callme(args):
+    python -c 'print("tree");print("car")'
+    print('one\ntwo\nthree')
+
+aliases['callme'] = _callme
+callme | grep t
+""",
+        """tree
+two
+three
+""",
+        0,
+    ),
+    # test $[] inside piped callable alias
+    pytest.param(
+        (
+            r"""
+def _callme(args):
+    $[python -c 'print("tree");print("car")']
+    print('one\ntwo\nthree')
+
+aliases['callme'] = _callme
+callme | grep t
+""",
+            """tree
+two
+three
+""",
+            0,
+        ),
+        marks=pytest.mark.xfail(reason="$[] does not send stdout through the pipe"),
+    ),
 ]
 
 if not ON_WINDOWS:
@@ -601,8 +708,8 @@ def test_eof_syntax_error():
     """Ensures syntax errors for EOF appear on last line."""
     script = "x = 1\na = (1, 0\n"
     out, err, rtn = run_xonsh(script, stderr=sp.PIPE)
-    assert ":0:0: EOF in multi-line statement" not in err
-    assert ":2:0: EOF in multi-line statement" in err
+    assert "line 0" not in err
+    assert "EOF in multi-line statement" in err and "line 2" in err
 
 
 @skip_if_no_xonsh
@@ -615,8 +722,8 @@ def test_open_quote_syntax_error():
         'echo "This is line 6"\n'
     )
     out, err, rtn = run_xonsh(script, stderr=sp.PIPE)
-    assert """:3:5: ('code: "This is line 3"',)""" not in err
-    assert ':5:4: "' in err
+    assert """('code: "This is line 3"',)""" not in err
+    assert "line 5" in err
     assert "SyntaxError:" in err
 
 
@@ -701,7 +808,7 @@ def test_subshells(cmd, fmt, exp):
 @pytest.mark.parametrize("cmd, exp", [("pwd", lambda: os.getcwd() + "\n")])
 def test_redirect_out_to_file(cmd, exp, tmpdir):
     outfile = tmpdir.mkdir("xonsh_test_dir").join("xonsh_test_file")
-    command = "{} > {}\n".format(cmd, outfile)
+    command = f"{cmd} > {outfile}\n"
     out, _, _ = run_xonsh(command)
     content = outfile.read()
     if callable(exp):
