@@ -144,6 +144,7 @@ def test_str_literal(check_ast):
 def test_bytes_literal(check_ast):
     check_ast('b"hello"')
     check_ast('B"hello"')
+    check_ast('b"hello" b"world"')
 
 
 def test_raw_literal(check_ast):
@@ -156,6 +157,16 @@ def test_f_literal(check_ast):
     check_ast('F"{yo}"', run=False)
 
 
+@pytest.mark.parametrize(
+    "first_prefix, second_prefix",
+    itertools.product(["", "f", "r", "fr"], repeat=2),
+)
+def test_string_literal_concat(first_prefix, second_prefix, check_ast):
+    check_ast(
+        first_prefix + r"'11{a}22\n'" + " " + second_prefix + r"'33{b}44\n'", run=False
+    )
+
+
 def test_f_env_var(check_xonsh_ast):
     check_xonsh_ast({}, 'f"{$HOME}"', run=False)
     check_xonsh_ast({}, "f'{$XONSH_DEBUG}'", run=False)
@@ -2202,6 +2213,16 @@ def test_path_fstring_literal(check_xonsh_ast):
     check_xonsh_ast({}, 'Fp"/foo{1+1}"', False)
 
 
+@pytest.mark.parametrize(
+    "first_prefix, second_prefix",
+    itertools.product(["p", "pf", "pr"], repeat=2),
+)
+def test_path_literal_concat(first_prefix, second_prefix, check_xonsh_ast):
+    check_xonsh_ast(
+        {}, first_prefix + r"'11{a}22\n'" + " " + second_prefix + r"'33{b}44\n'", False
+    )
+
+
 def test_dollar_name(check_xonsh_ast):
     check_xonsh_ast({"WAKKA": 42}, "$WAKKA")
 
@@ -2376,8 +2397,11 @@ def test_ls_regex(check_xonsh_ast):
     check_xonsh_ast({}, "$(ls `[Ff]+i*LE` -l)", False)
 
 
-def test_backtick(check_xonsh_ast):
-    check_xonsh_ast({}, "print(`.*`)", False)
+@pytest.mark.parametrize("p", ["", "p"])
+@pytest.mark.parametrize("f", ["", "f"])
+@pytest.mark.parametrize("glob_type", ["", "r", "g"])
+def test_backtick(p, f, glob_type, check_xonsh_ast):
+    check_xonsh_ast({}, f"print({p}{f}{glob_type}`.*`)", False)
 
 
 def test_ls_regex_octothorpe(check_xonsh_ast):
@@ -2388,10 +2412,6 @@ def test_ls_explicitregex(check_xonsh_ast):
     check_xonsh_ast({}, "$(ls r`[Ff]+i*LE` -l)", False)
 
 
-def test_rbacktick(check_xonsh_ast):
-    check_xonsh_ast({}, "print(r`.*`)", False)
-
-
 def test_ls_explicitregex_octothorpe(check_xonsh_ast):
     check_xonsh_ast({}, "$(ls r`#[Ff]+i*LE` -l)", False)
 
@@ -2400,22 +2420,6 @@ def test_ls_glob(check_xonsh_ast):
     check_xonsh_ast({}, "$(ls g`[Ff]+i*LE` -l)", False)
 
 
-def test_gbacktick(check_xonsh_ast):
-    check_xonsh_ast({}, "print(g`.*`)", False)
-
-
-def test_pbacktrick(check_xonsh_ast):
-    check_xonsh_ast({}, "print(p`.*`)", False)
-
-
-def test_pgbacktick(check_xonsh_ast):
-    check_xonsh_ast({}, "print(pg`.*`)", False)
-
-
-def test_prbacktick(check_xonsh_ast):
-    check_xonsh_ast({}, "print(pr`.*`)", False)
-
-
 def test_ls_glob_octothorpe(check_xonsh_ast):
     check_xonsh_ast({}, "$(ls g`#[Ff]+i*LE` -l)", False)
 
@@ -3279,6 +3283,14 @@ def test_syntax_error_lambda_posonly_nondefault_follows_default(parser):
         parser.parse("lambda x, y=1, /, z: x", mode="exec")
 
 
+@pytest.mark.parametrize(
+    "first_prefix, second_prefix", itertools.permutations(["", "p", "b"], 2)
+)
+def test_syntax_error_literal_concat_different(first_prefix, second_prefix, parser):
+    with pytest.raises(SyntaxError):
+        parser.parse(f"{first_prefix}'hello' {second_prefix}'world'")
+
+
 def test_get_repo_url(parser):
     parser.parse(
         "def get_repo_url():\n"
