@@ -1,99 +1,94 @@
-# -*- coding: utf-8 -*-
 """Tests xonsh tools."""
 import datetime as dt
 import os
 import pathlib
 import stat
-from tempfile import TemporaryDirectory
 import warnings
+from tempfile import TemporaryDirectory
 
 import pytest
 
 from xonsh import __version__
-from xonsh.platform import (
-    ON_WINDOWS,
-    HAS_PYGMENTS,
-)
 from xonsh.lexer import Lexer
-
+from xonsh.platform import HAS_PYGMENTS, ON_WINDOWS
+from xonsh.pytest.tools import skip_if_on_windows
 from xonsh.tools import (
     EnvPath,
+    all_permutations,
     always_false,
     always_true,
     argvquote,
+    balanced_parens,
     bool_or_int_to_str,
     bool_or_none_to_str,
     bool_to_str,
     check_for_partial_string,
+    check_quotes,
+    deprecated,
     dynamic_cwd_tuple_to_str,
+    ends_with_colon_token,
     ensure_slice,
     ensure_string,
-    path_to_str,
+    ensure_timestamp,
     env_path_to_str,
     escape_windows_cmd_string,
     executables_in,
+    expand_case_matching,
     expand_path,
+    expandvars,
     find_next_break,
+    get_line_continuation,
+    get_logical_line,
+    get_portions,
+    iglobpath,
+    is_balanced,
     is_bool,
     is_bool_or_int,
     is_bool_or_none,
     is_callable,
+    is_completion_mode,
+    is_completions_display_value,
     is_dynamic_cwd_width,
-    is_path,
     is_env_path,
     is_float,
     is_int,
+    is_int_as_str,
     is_logfile_opt,
-    is_string_or_callable,
-    logfile_opt_to_str,
-    str_to_path,
-    str_to_env_path,
+    is_nonstring_seq_of_strings,
+    is_path,
+    is_regex,
+    is_slice_as_str,
     is_string,
+    is_string_or_callable,
+    is_string_seq,
+    is_tok_color_dict,
+    is_writable_file,
+    logfile_opt_to_str,
+    path_to_str,
+    pathsep_to_seq,
+    pathsep_to_set,
+    pathsep_to_upper_seq,
+    register_custom_style,
+    replace_logical_line,
+    seq_to_pathsep,
+    seq_to_upper_pathsep,
+    set_to_pathsep,
+    simple_random_choice,
+    str_to_env_path,
+    str_to_path,
+    subexpr_before_unbalanced,
     subexpr_from_unbalanced,
     subproc_toks,
+    swap_values,
     to_bool,
     to_bool_or_int,
     to_bool_or_none,
-    to_int_or_none,
-    to_dynamic_cwd_tuple,
-    to_logfile_opt,
-    pathsep_to_set,
-    set_to_pathsep,
-    is_string_seq,
-    pathsep_to_seq,
-    seq_to_pathsep,
-    is_nonstring_seq_of_strings,
-    pathsep_to_upper_seq,
-    seq_to_upper_pathsep,
-    is_int_as_str,
-    is_slice_as_str,
-    ensure_timestamp,
-    get_portions,
-    is_balanced,
-    subexpr_before_unbalanced,
-    swap_values,
-    get_line_continuation,
-    get_logical_line,
-    replace_logical_line,
-    check_quotes,
-    deprecated,
-    is_writable_file,
-    balanced_parens,
-    ends_with_colon_token,
-    iglobpath,
-    all_permutations,
-    register_custom_style,
-    simple_random_choice,
-    is_completion_mode,
     to_completion_mode,
-    is_completions_display_value,
     to_completions_display_value,
-    expand_case_matching,
-    expandvars,
+    to_dynamic_cwd_tuple,
+    to_int_or_none,
+    to_logfile_opt,
 )
-from xonsh.environ import Env
-
-from tools import skip_if_on_windows
 
 LEXER = Lexer()
 LEXER.build()
@@ -128,49 +123,49 @@ def test_subproc_toks_ls_l():
 
 def test_subproc_toks_git():
     s = 'git commit -am "hello doc"'
-    exp = "![{0}]".format(s)
+    exp = f"![{s}]"
     obs = subproc_toks(s, lexer=LEXER, returnline=True)
     assert exp == obs
 
 
 def test_subproc_toks_git_semi():
     s = 'git commit -am "hello doc"'
-    exp = "![{0}];".format(s)
+    exp = f"![{s}];"
     obs = subproc_toks(s + ";", lexer=LEXER, returnline=True)
     assert exp == obs
 
 
 def test_subproc_toks_git_nl():
     s = 'git commit -am "hello doc"'
-    exp = "![{0}]\n".format(s)
+    exp = f"![{s}]\n"
     obs = subproc_toks(s + "\n", lexer=LEXER, returnline=True)
     assert exp == obs
 
 
 def test_bash_macro():
     s = "bash -c ! export var=42; echo $var"
-    exp = "![{0}]\n".format(s)
+    exp = f"![{s}]\n"
     obs = subproc_toks(s + "\n", lexer=LEXER, returnline=True)
     assert exp == obs
 
 
 def test_python_macro():
     s = 'python -c ! import os; print(os.path.abspath("/"))'
-    exp = "![{0}]\n".format(s)
+    exp = f"![{s}]\n"
     obs = subproc_toks(s + "\n", lexer=LEXER, returnline=True)
     assert exp == obs
 
 
 def test_subproc_toks_indent_ls():
     s = "ls -l"
-    exp = INDENT + "![{0}]".format(s)
+    exp = INDENT + f"![{s}]"
     obs = subproc_toks(INDENT + s, mincol=len(INDENT), lexer=LEXER, returnline=True)
     assert exp == obs
 
 
 def test_subproc_toks_indent_ls_nl():
     s = "ls -l"
-    exp = INDENT + "![{0}]\n".format(s)
+    exp = INDENT + f"![{s}]\n"
     obs = subproc_toks(
         INDENT + s + "\n", mincol=len(INDENT), lexer=LEXER, returnline=True
     )
@@ -179,28 +174,28 @@ def test_subproc_toks_indent_ls_nl():
 
 def test_subproc_toks_indent_ls_no_min():
     s = "ls -l"
-    exp = INDENT + "![{0}]".format(s)
+    exp = INDENT + f"![{s}]"
     obs = subproc_toks(INDENT + s, lexer=LEXER, returnline=True)
     assert exp == obs
 
 
 def test_subproc_toks_indent_ls_no_min_nl():
     s = "ls -l"
-    exp = INDENT + "![{0}]\n".format(s)
+    exp = INDENT + f"![{s}]\n"
     obs = subproc_toks(INDENT + s + "\n", lexer=LEXER, returnline=True)
     assert exp == obs
 
 
 def test_subproc_toks_indent_ls_no_min_semi():
     s = "ls"
-    exp = INDENT + "![{0}];".format(s)
+    exp = INDENT + f"![{s}];"
     obs = subproc_toks(INDENT + s + ";", lexer=LEXER, returnline=True)
     assert exp == obs
 
 
 def test_subproc_toks_indent_ls_no_min_semi_nl():
     s = "ls"
-    exp = INDENT + "![{0}];\n".format(s)
+    exp = INDENT + f"![{s}];\n"
     obs = subproc_toks(INDENT + s + ";\n", lexer=LEXER, returnline=True)
     assert exp == obs
 
@@ -208,7 +203,7 @@ def test_subproc_toks_indent_ls_no_min_semi_nl():
 def test_subproc_toks_ls_comment():
     s = "ls -l"
     com = "  # lets list"
-    exp = "![{0}]{1}".format(s, com)
+    exp = f"![{s}]{com}"
     obs = subproc_toks(s + com, lexer=LEXER, returnline=True)
     assert exp == obs
 
@@ -216,7 +211,7 @@ def test_subproc_toks_ls_comment():
 def test_subproc_toks_ls_42_comment():
     s = "ls 42"
     com = "  # lets list"
-    exp = "![{0}]{1}".format(s, com)
+    exp = f"![{s}]{com}"
     obs = subproc_toks(s + com, lexer=LEXER, returnline=True)
     assert exp == obs
 
@@ -224,7 +219,7 @@ def test_subproc_toks_ls_42_comment():
 def test_subproc_toks_ls_str_comment():
     s = 'ls "wakka"'
     com = "  # lets list"
-    exp = "![{0}]{1}".format(s, com)
+    exp = f"![{s}]{com}"
     obs = subproc_toks(s + com, lexer=LEXER, returnline=True)
     assert exp == obs
 
@@ -233,7 +228,7 @@ def test_subproc_toks_indent_ls_comment():
     ind = "    "
     s = "ls -l"
     com = "  # lets list"
-    exp = "{0}![{1}]{2}".format(ind, s, com)
+    exp = f"{ind}![{s}]{com}"
     obs = subproc_toks(ind + s + com, lexer=LEXER, returnline=True)
     assert exp == obs
 
@@ -242,7 +237,7 @@ def test_subproc_toks_indent_ls_str():
     ind = "    "
     s = 'ls "wakka"'
     com = "  # lets list"
-    exp = "{0}![{1}]{2}".format(ind, s, com)
+    exp = f"{ind}![{s}]{com}"
     obs = subproc_toks(ind + s + com, lexer=LEXER, returnline=True)
     assert exp == obs
 
@@ -250,8 +245,8 @@ def test_subproc_toks_indent_ls_str():
 def test_subproc_toks_ls_l_semi_ls_first():
     lsdl = "ls -l"
     ls = "ls"
-    s = "{0}; {1}".format(lsdl, ls)
-    exp = "![{0}]; {1}".format(lsdl, ls)
+    s = f"{lsdl}; {ls}"
+    exp = f"![{lsdl}]; {ls}"
     obs = subproc_toks(s, lexer=LEXER, maxcol=6, returnline=True)
     assert exp == obs
 
@@ -259,8 +254,8 @@ def test_subproc_toks_ls_l_semi_ls_first():
 def test_subproc_toks_ls_l_semi_ls_second():
     lsdl = "ls -l"
     ls = "ls"
-    s = "{0}; {1}".format(lsdl, ls)
-    exp = "{0}; ![{1}]".format(lsdl, ls)
+    s = f"{lsdl}; {ls}"
+    exp = f"{lsdl}; ![{ls}]"
     obs = subproc_toks(s, lexer=LEXER, mincol=7, returnline=True)
     assert exp == obs
 
@@ -268,8 +263,8 @@ def test_subproc_toks_ls_l_semi_ls_second():
 def test_subproc_toks_hello_mom_first():
     fst = "echo 'hello'"
     sec = "echo 'mom'"
-    s = "{0}; {1}".format(fst, sec)
-    exp = "![{0}]; {1}".format(fst, sec)
+    s = f"{fst}; {sec}"
+    exp = f"![{fst}]; {sec}"
     obs = subproc_toks(s, lexer=LEXER, maxcol=len(fst) + 1, returnline=True)
     assert exp == obs
 
@@ -277,8 +272,8 @@ def test_subproc_toks_hello_mom_first():
 def test_subproc_toks_hello_mom_second():
     fst = "echo 'hello'"
     sec = "echo 'mom'"
-    s = "{0}; {1}".format(fst, sec)
-    exp = "{0}; ![{1}]".format(fst, sec)
+    s = f"{fst}; {sec}"
+    exp = f"{fst}; ![{sec}]"
     obs = subproc_toks(s, lexer=LEXER, mincol=len(fst), returnline=True)
     assert exp == obs
 
@@ -305,7 +300,7 @@ def test_subproc_toks_hello_bad_trailing_triple_quotes():
 
 def test_subproc_toks_hello_mom_triple_quotes_nl():
     s = 'echo """hello\nmom"""'
-    exp = "![{0}]".format(s)
+    exp = f"![{s}]"
     obs = subproc_toks(s, lexer=LEXER, returnline=True)
     assert exp == obs
 
@@ -360,44 +355,44 @@ def test_subproc_toks_semicolon_only():
 
 def test_subproc_toks_pyeval():
     s = "echo @(1+1)"
-    exp = "![{0}]".format(s)
+    exp = f"![{s}]"
     obs = subproc_toks(s, lexer=LEXER, returnline=True)
     assert exp == obs
 
 
 def test_subproc_toks_pyeval_multiline_string():
     s = 'echo @("""hello\nmom""")'
-    exp = "![{0}]".format(s)
+    exp = f"![{s}]"
     obs = subproc_toks(s, lexer=LEXER, returnline=True)
     assert exp == obs
 
 
 def test_subproc_toks_twopyeval():
     s = "echo @(1+1) @(40 + 2)"
-    exp = "![{0}]".format(s)
+    exp = f"![{s}]"
     obs = subproc_toks(s, lexer=LEXER, returnline=True)
     assert exp == obs
 
 
 def test_subproc_toks_pyeval_parens():
     s = "echo @(1+1)"
-    inp = "({0})".format(s)
-    exp = "(![{0}])".format(s)
+    inp = f"({s})"
+    exp = f"(![{s}])"
     obs = subproc_toks(inp, lexer=LEXER, returnline=True)
     assert exp == obs
 
 
 def test_subproc_toks_twopyeval_parens():
     s = "echo @(1+1) @(40+2)"
-    inp = "({0})".format(s)
-    exp = "(![{0}])".format(s)
+    inp = f"({s})"
+    exp = f"(![{s}])"
     obs = subproc_toks(inp, lexer=LEXER, returnline=True)
     assert exp == obs
 
 
 def test_subproc_toks_pyeval_nested():
     s = "echo @(min(1, 42))"
-    exp = "![{0}]".format(s)
+    exp = f"![{s}]"
     obs = subproc_toks(s, lexer=LEXER, returnline=True)
     assert exp == obs
 
@@ -423,65 +418,65 @@ def test_subproc_toks_pyeval_nested():
 )
 def test_subproc_toks_and_or(phrase):
     s = "echo " + phrase
-    exp = "![{0}]".format(s)
+    exp = f"![{s}]"
     obs = subproc_toks(s, lexer=LEXER, returnline=True)
     assert exp == obs
 
 
 def test_subproc_toks_pyeval_nested_parens():
     s = "echo @(min(1, 42))"
-    inp = "({0})".format(s)
-    exp = "(![{0}])".format(s)
+    inp = f"({s})"
+    exp = f"(![{s}])"
     obs = subproc_toks(inp, lexer=LEXER, returnline=True)
     assert exp == obs
 
 
 def test_subproc_toks_capstdout():
     s = "echo $(echo bat)"
-    exp = "![{0}]".format(s)
+    exp = f"![{s}]"
     obs = subproc_toks(s, lexer=LEXER, returnline=True)
     assert exp == obs
 
 
 def test_subproc_toks_capproc():
     s = "echo !(echo bat)"
-    exp = "![{0}]".format(s)
+    exp = f"![{s}]"
     obs = subproc_toks(s, lexer=LEXER, returnline=True)
     assert exp == obs
 
 
 def test_subproc_toks_pyeval_redirect():
     s = 'echo @("foo") > bar'
-    inp = "{0}".format(s)
-    exp = "![{0}]".format(s)
+    inp = f"{s}"
+    exp = f"![{s}]"
     obs = subproc_toks(inp, lexer=LEXER, returnline=True)
     assert exp == obs
 
 
 def test_subproc_toks_greedy_parens():
     s = "(sort)"
-    exp = "![{0}]".format(s)
+    exp = f"![{s}]"
     obs = subproc_toks(s, lexer=LEXER, returnline=True, greedy=True)
     assert exp == obs
 
 
 def test_subproc_toks_greedy_parens_inp():
     s = "(sort) < input.txt"
-    exp = "![{0}]".format(s)
+    exp = f"![{s}]"
     obs = subproc_toks(s, lexer=LEXER, returnline=True, greedy=True)
     assert exp == obs
 
 
 def test_subproc_toks_greedy_parens_statements():
     s = '(echo "abc"; sleep 1; echo "def")'
-    exp = "![{0}]".format(s)
+    exp = f"![{s}]"
     obs = subproc_toks(s, lexer=LEXER, returnline=True, greedy=True)
     assert exp == obs
 
 
 def test_subproc_toks_greedy_parens_statements_with_grep():
     s = '(echo "abc"; sleep 1; echo "def") | grep'
-    exp = "![{0}]".format(s)
+    exp = f"![{s}]"
     obs = subproc_toks(s, lexer=LEXER, returnline=True, greedy=True)
     assert exp == obs
 
@@ -537,7 +532,7 @@ mom"""
 
 
 @pytest.mark.parametrize("src, idx, exp_line, exp_n", LOGICAL_LINE_CASES)
-def test_get_logical_line(src, idx, exp_line, exp_n, xonsh_builtins):
+def test_get_logical_line(src, idx, exp_line, exp_n, xession):
     lines = src.splitlines()
     line, n, start = get_logical_line(lines, idx)
     assert exp_line == line
@@ -545,7 +540,7 @@ def test_get_logical_line(src, idx, exp_line, exp_n, xonsh_builtins):
 
 
 @pytest.mark.parametrize("src, idx, exp_line, exp_n", LOGICAL_LINE_CASES)
-def test_replace_logical_line(src, idx, exp_line, exp_n, xonsh_builtins):
+def test_replace_logical_line(src, idx, exp_line, exp_n, xession):
     lines = src.splitlines()
     logical = exp_line
     while idx > 0 and lines[idx - 1].endswith("\\"):
@@ -1424,6 +1419,7 @@ def test_is_logfile_opt(inp, exp):
             else None,
             marks=skip_if_on_windows,
         ),
+        ("~/log", os.path.expanduser("~/log")),
     ],
 )
 def test_to_logfile_opt(inp, exp):
@@ -1572,7 +1568,7 @@ def test_executables_in(xession):
     with TemporaryDirectory() as test_path:
         for _type in types:
             for executable in executables:
-                fname = "%s_%s" % (_type, executable)
+                fname = f"{_type}_{executable}"
                 if _type == "none":
                     continue
                 if _type == "file" and executable:
@@ -1629,8 +1625,8 @@ def test_expand_case_matching(inp, exp):
         ("$unk $foo $unk $spam $unk", "$unk bar $unk eggs $unk"),
         ("$an_int$spam$a_bool", "42eggsTrue"),
         ("$unk$an_int$spam$a_bool", "$unk42eggsTrue"),
-        ("bar$foo$spam$foo $an_int $none", "barbareggsbar 42 None"),
-        ("$unk bar$foo$spam$foo $an_int $none", "$unk barbareggsbar 42 None"),
+        ("bar$foo$spam$foo $an_int $none", "barbareggsbar 42 "),
+        ("$unk bar$foo$spam$foo $an_int $none", "$unk barbareggsbar 42 "),
         ("$foo/bar", "bar/bar"),
         ("$unk/$foo/bar", "$unk/bar/bar"),
         ("${'foo'} $spam", "bar eggs"),
@@ -1652,10 +1648,9 @@ def test_expand_case_matching(inp, exp):
 )
 def test_expandvars(inp, exp, xession):
     """Tweaked for xonsh cases from CPython `test_genericpath.py`"""
-    env = Env(
-        {"foo": "bar", "spam": "eggs", "a_bool": True, "an_int": 42, "none": None}
+    xession.env.update(
+        dict({"foo": "bar", "spam": "eggs", "a_bool": True, "an_int": 42, "none": None})
     )
-    xession.env = env
     assert expandvars(inp) == exp
 
 
@@ -1691,7 +1686,7 @@ def test_ensure_timestamp(inp, fmt, exp, xession):
         ("~/$foo", True, "/bar"),
         ("~/test/$a_bool", True, "/test/True"),
         ("~/test/$an_int", True, "/test/42"),
-        ("~/test/$none", True, "/test/None"),
+        ("~/test/$none", True, "/test/"),
         ("~/$foo", False, "/$foo"),
     ],
 )
@@ -1700,9 +1695,8 @@ def test_expand_path(expand_user, inp, expand_env_vars, exp_end, xession):
         inp = inp.replace("/", os.sep)
         exp_end = exp_end.replace("/", os.sep)
 
-    env = Env({"foo": "bar", "a_bool": True, "an_int": 42, "none": None})
-    env["EXPAND_ENV_VARS"] = expand_env_vars
-    xession.env = env
+    xession.env.update({"foo": "bar", "a_bool": True, "an_int": 42, "none": None})
+    xession.env["EXPAND_ENV_VARS"] = expand_env_vars
 
     path = expand_path(inp, expand_user=expand_user)
 
@@ -1999,3 +1993,46 @@ def test_to_completions_display_value_fail(val):
     with pytest.warns(RuntimeWarning):
         obs = to_completions_display_value(val)
         assert obs == "multi"
+
+
+def test_is_regex_true():
+    assert is_regex("cat")
+
+
+def test_is_regex_false():
+    assert not is_regex("**")
+
+
+from xonsh.style_tools import Token
+
+
+@pytest.mark.parametrize(
+    "val, exp",
+    [
+        (
+            {
+                Token.Literal.String: "bold ansigreen",
+                "Token.Name.Tag": "underline ansiblue",
+            },
+            True,
+        ),
+        (
+            {
+                Token.Literal.String: "bold ansigreen",
+                1: "bold ansigreen",
+            },
+            False,
+        ),
+        ({1: "bold ansigreen"}, False),
+        (
+            {Token.Literal.String: "123"},
+            False,
+        ),
+        (
+            {"Token.Name.Tag": 123},
+            False,
+        ),
+    ],
+)
+def test_is_tok_color_dict(val, exp):
+    assert is_tok_color_dict(val) == exp
