@@ -1,126 +1,128 @@
-# -*- coding: utf-8 -*-
-"""The xonsh abstract syntax tree node."""
+"""Abstract Syntax Tree handler"""
 # These are imported into our module namespace for the benefit of parser.py.
 # pylint: disable=unused-import
+import itertools
 import sys
-import builtins
-from ast import (
-    Module,
-    Num,
-    Expr,
-    Str,
-    Bytes,
-    UnaryOp,
-    UAdd,
-    USub,
-    Invert,
-    BinOp,
-    Add,
-    Sub,
-    Mult,
-    Div,
-    FloorDiv,
-    Mod,
-    Pow,
-    Compare,
-    Lt,
-    Gt,
-    LtE,
-    GtE,
-    Eq,
-    NotEq,
-    In,
-    NotIn,
-    Is,
-    IsNot,
-    Not,
-    BoolOp,
-    Or,
-    And,
-    Subscript,
-    Load,
-    Slice,
-    ExtSlice,
-    List,
-    Tuple,
-    Set,
-    Dict,
-    AST,
-    NameConstant,
-    Name,
-    GeneratorExp,
-    Store,
-    comprehension,
-    ListComp,
-    SetComp,
-    DictComp,
-    Assign,
-    AugAssign,
-    BitXor,
-    BitAnd,
-    BitOr,
-    LShift,
-    RShift,
-    Assert,
-    Delete,
-    Del,
-    Pass,
-    Raise,
-    Import,
-    alias,
-    ImportFrom,
-    Continue,
-    Break,
-    Yield,
-    YieldFrom,
-    Return,
-    IfExp,
-    Lambda,
-    arguments,
-    arg,
-    Call,
-    keyword,
-    Attribute,
-    Global,
-    Nonlocal,
-    If,
-    While,
-    For,
-    withitem,
-    With,
-    Try,
-    ExceptHandler,
-    FunctionDef,
-    ClassDef,
-    Starred,
-    NodeTransformer,
-    Interactive,
-    Expression,
-    Index,
-    literal_eval,
-    dump,
-    walk,
-    increment_lineno,
-)
-from ast import Ellipsis as EllipsisNode
 
 # pylint: enable=unused-import
 import textwrap
-import itertools
-
-from xonsh.tools import subproc_toks, find_next_break, get_logical_line
-
 from ast import (
-    MatMult,
+    AST,
+    Add,
+    And,
+    AnnAssign,
+    Assert,
+    Assign,
+    AsyncFor,
     AsyncFunctionDef,
     AsyncWith,
-    AsyncFor,
+    Attribute,
+    AugAssign,
     Await,
-    JoinedStr,
+    BinOp,
+    BitAnd,
+    BitOr,
+    BitXor,
+    BoolOp,
+    Break,
+    Bytes,
+    Call,
+    ClassDef,
+    Compare,
+    Constant,
+    Continue,
+    Del,
+    Delete,
+    Dict,
+    DictComp,
+    Div,
+)
+from ast import Ellipsis as EllipsisNode
+from ast import (
+    Eq,
+    ExceptHandler,
+    Expr,
+    Expression,
+    ExtSlice,
+    FloorDiv,
+    For,
     FormattedValue,
-    AnnAssign,
+    FunctionDef,
+    GeneratorExp,
+    Global,
+    Gt,
+    GtE,
+    If,
+    IfExp,
+    Import,
+    ImportFrom,
+    In,
+    Index,
+    Interactive,
+    Invert,
+    Is,
+    IsNot,
+    JoinedStr,
+    Lambda,
+    List,
+    ListComp,
+    Load,
+    LShift,
+    Lt,
+    LtE,
+    MatMult,
+    Mod,
+    Module,
+    Mult,
+    Name,
+    NameConstant,
+    NodeTransformer,
+    Nonlocal,
+    Not,
+    NotEq,
+    NotIn,
+    Num,
+    Or,
+    Pass,
+    Pow,
+    Raise,
+    Return,
+    RShift,
+    Set,
+    SetComp,
+    Slice,
+    Starred,
+    Store,
+    Str,
+    Sub,
+    Subscript,
+    Try,
+    Tuple,
+    UAdd,
+    UnaryOp,
+    USub,
+    While,
+    With,
+    Yield,
+    YieldFrom,
+    alias,
+    arg,
+    arguments,
+    comprehension,
+    dump,
+    increment_lineno,
+    keyword,
+    literal_eval,
+    walk,
+    withitem,
 )
 
+from xonsh.built_ins import XSH
+from xonsh.platform import PYTHON_VERSION_INFO
+from xonsh.tools import find_next_break, get_logical_line, subproc_toks
+
+if PYTHON_VERSION_INFO > (3, 8):
+    from ast import NamedExpr  # type:ignore
 
 STATEMENTS = (
     FunctionDef,
@@ -162,8 +164,10 @@ def leftmostname(node):
         rtn = leftmostname(node.operand)
     elif isinstance(node, BoolOp):
         rtn = leftmostname(node.values[0])
-    elif isinstance(node, (Assign, AnnAssign)):
+    elif isinstance(node, Assign):
         rtn = leftmostname(node.targets[0])
+    elif isinstance(node, AnnAssign):
+        rtn = leftmostname(node.target)
     elif isinstance(node, (Str, Bytes, JoinedStr)):
         # handles case of "./my executable"
         rtn = leftmostname(node.s)
@@ -307,9 +311,9 @@ def isexpression(node, ctx=None, *args, **kwargs):
     # parse string to AST
     if isinstance(node, str):
         node = node if node.endswith("\n") else node + "\n"
-        ctx = builtins.__xonsh__.ctx if ctx is None else ctx
-        node = builtins.__xonsh__.execer.parse(node, ctx, *args, **kwargs)
-    # determin if expresission-like enough
+        ctx = XSH.ctx if ctx is None else ctx
+        node = XSH.execer.parse(node, ctx, *args, **kwargs)
+    # determine if expression-like enough
     if isinstance(node, (Expr, Expression)):
         isexpr = True
     elif isinstance(node, Module) and len(node.body) == 1:
@@ -332,7 +336,7 @@ class CtxAwareTransformer(NodeTransformer):
         parser : xonsh.Parser
             A parse instance to try to parse subprocess statements with.
         """
-        super(CtxAwareTransformer, self).__init__()
+        super().__init__()
         self.parser = parser
         self.input = None
         self.contexts = []
@@ -349,7 +353,7 @@ class CtxAwareTransformer(NodeTransformer):
         ----------
         node : ast.AST
             A syntax tree to transform.
-        input : str
+        inp : str
             The input code in string format.
         ctx : dict
             The root context to use.
@@ -413,7 +417,7 @@ class CtxAwareTransformer(NodeTransformer):
             returnline=False,
             lexer=self.parser.lexer,
         )
-        if spline is None or spline != "![{}]".format(line[mincol:maxcol].strip()):
+        if spline is None or spline != f"![{line[mincol:maxcol].strip()}]":
             # failed to get something consistent, try greedy wrap
             spline = subproc_toks(
                 line,
@@ -430,7 +434,7 @@ class CtxAwareTransformer(NodeTransformer):
                 spline,
                 mode=self.mode,
                 filename=self.filename,
-                debug_level=(self.debug_level > 2),
+                debug_level=(self.debug_level >= 2),
             )
             newnode = newnode.body
             if not isinstance(newnode, AST):
@@ -438,7 +442,7 @@ class CtxAwareTransformer(NodeTransformer):
                 newnode = newnode[0]
             increment_lineno(newnode, n=node.lineno - 1)
             newnode.col_offset = node.col_offset
-            if self.debug_level > 1:
+            if self.debug_level >= 1:
                 msg = "{0}:{1}:{2}{3} - {4}\n" "{0}:{1}:{2}{3} + {5}"
                 mstr = "" if maxcol is None else ":" + str(maxcol)
                 msg = msg.format(self.filename, node.lineno, mincol, mstr, line, spline)
@@ -515,6 +519,11 @@ class CtxAwareTransformer(NodeTransformer):
                 node.values[i] = self.try_subproc_toks(val, strip_expr=True)
         return node
 
+    def visit_comprehension(self, node):
+        """Handles visiting list comprehensions, set comprehensions,
+        dictionary comprehensions, and generator expressions."""
+        return node  # do not descend into any comprehensions
+
     #
     # Context aggregator visitors
     #
@@ -536,7 +545,10 @@ class CtxAwareTransformer(NodeTransformer):
         self.ctxupdate(ups)
         return node
 
-    visit_AnnAssign = visit_Assign
+    def visit_AnnAssign(self, node):
+        """Handle visiting an annotated assignment statement."""
+        self.ctxadd(leftmostname(node.target))
+        return node
 
     def visit_Import(self, node):
         """Handle visiting a import statement."""
@@ -628,7 +640,7 @@ def pdump(s, **kwargs):
     lens = len(s) + 1
     if lens == 1:
         return s
-    i = min([s.find(o) % lens for o in openers])
+    i = min(s.find(o) % lens for o in openers)
     if i == lens - 1:
         return s
     closer = closers[openers.find(s[i])]
