@@ -6,12 +6,16 @@ Examples
 """
 
 import argparse as ap
+import functools
 import inspect
 import os
 import sys
 import typing as tp
 from collections import defaultdict
 
+from xonsh.built_ins import XSH
+from xonsh.completers.tools import RichCompletion
+
 TYPING_ANNOTATED_AVAILABLE = False
 """One can import ``Annotated`` from this module
 which adds a stub when it is not available in ``typing``/``typing_extensions`` modules."""
@@ -61,7 +65,7 @@ def Arg(
 class NumpyDoc:
     """Represent parsed function docstring"""
 
-    def __init__(self, func):
+    def __init__(self, func, prefix_chars="-", follow_wraps=True):
         """Parse the function docstring and return its help content
 
         Parameters
@@ -70,6 +74,9 @@ class NumpyDoc:
             a callable/object that holds docstring
         """
 
+        if follow_wraps and isinstance(func, functools.partial):
+            func = func.func
+
         doc: str = inspect.getdoc(func) or ""
         self.description, rest = self.get_func_doc(doc)
 
@@ -80,7 +87,7 @@ class NumpyDoc:
             parts = [st.strip() for st in head.split(":")]
             if len(parts) == 2:
                 name, flag = parts
-                if flag and flag.startswith("-"):
+                if flag and any(map(flag.startswith, prefix_chars)):
                     self.flags[name] = [st.strip() for st in flag.split(",")]
             else:
                 name = parts[0]
@@ -140,7 +147,7 @@ def _get_args_kwargs(annot: tp.Any) -> tp.Tuple[tp.Sequence[str], tp.Dict[str, t
         if hasattr(annot, "__metadata__"):
             args, kwargs = annot.__metadata__[0]
         else:
-            from typing_extensions import get_args
+            from typing import get_args
 
             _, (args, kwargs) = get_args(annot)
 
@@ -151,15 +158,20 @@ def _get_args_kwargs(annot: tp.Any) -> tp.Tuple[tp.Sequence[str], tp.Dict[str, t
 
 
 def add_args(
-    parser: ap.ArgumentParser, func: tp.Callable, allowed_params=None, doc=None
+    parser: ap.ArgumentParser,
+    func: tp.Callable,
+    allowed_params=None,
+    doc=None,
 ) -> None:
     """Using the function's annotation add arguments to the parser
-    param:Arg(*args, **kw) -> parser.add_argument(*args, *kw)
+
+    basically converts ``def fn(param : Arg(*args, **kw), ...): ...``
+        -> into equivalent ``parser.add_argument(*args, *kw)`` call.
     """
 
     # call this function when this sub-command is selected
     parser.set_defaults(**{_FUNC_NAME: func})
-    doc = doc or NumpyDoc(func)
+    doc = doc or NumpyDoc(func, parser.prefix_chars)
     sign = inspect.signature(func)
     for name, param in sign.parameters.items():
         if name.startswith("_") or (
@@ -167,7 +179,7 @@ def add_args(
         ):
             continue
         flags, kwargs = _get_args_kwargs(param.annotation)
-        if not flags:  # load from docstring
+        if (not flags) and (name in doc.flags):  # load from docstring
             flags = doc.flags.get(name)
 
         if flags:  # optional argument. eg. --option
@@ -175,6 +187,15 @@ def add_args(
         else:  # positional argument
             flags = [name]
 
+            # checks for optional positional arg
+            if (
+                (inspect.Parameter.empty != param.default)
+                and (param.default is None)
+                and ("nargs" not in kwargs)
+                and ("action" not in kwargs)
+            ):
+                kwargs.setdefault("nargs", "?")
+
         if inspect.Parameter.empty != param.default:
             kwargs.setdefault("default", param.default)
 
@@ -193,12 +214,33 @@ def add_args(
         kwargs.setdefault("help", doc.params.get(name))
 
         completer = kwargs.pop("completer", None)
+        cls_inst = getattr(func, "__self__", None)
+
+        if cls_inst and isinstance(cls_inst, ArgParserAlias):
+            flags, kwargs = cls_inst.hook_pre_add_argument(
+                param=name, func=func, flags=flags, kwargs=kwargs
+            )
+
         action = parser.add_argument(*flags, **kwargs)
+
         if completer:
             action.completer = completer  # type: ignore
+
+        if cls_inst and isinstance(cls_inst, ArgParserAlias):
+            cls_inst.hook_post_add_argument(
+                parser=parser, action=action, param=name, func=func
+            )
+
         action.help = action.help or ""
-        if action.default and "%(default)s" not in action.help:
-            action.help += os.linesep + " (default: %(default)s)"
+        # Don't show default when
+        # 1. None : No value is given for the option
+        # 2. bool : in case of flags the default is opposite of the flag's meaning
+        if (
+            action.default
+            and (not isinstance(action.default, bool))
+            and ("%(default)s" not in action.help)
+        ):
+            action.help += os.linesep + " (default: '%(default)s')"
         if action.type and "%(type)s" not in action.help:
             action.help += " (type: %(type)s)"
 
@@ -259,7 +301,7 @@ class RstHelpFormatter(ap.RawTextHelpFormatter):
         return text
 
     def _format_usage(self, usage, actions, groups, prefix):
-        from pygments.token import Name, Generic
+        from pygments.token import Generic, Name
 
         text = super()._format_usage(usage, actions, groups, prefix)
         parts = text.split(self._prog, maxsplit=1)
@@ -279,7 +321,6 @@ class RstHelpFormatter(ap.RawTextHelpFormatter):
 
 
 def get_argparse_formatter_class():
-    from xonsh.built_ins import XSH
     from xonsh.platform import HAS_PYGMENTS
 
     if (
@@ -384,6 +425,152 @@ def dispatch(parser: ap.ArgumentParser, args=None, lenient=False, **ns):
     return _dispatch_func(func, ns)
 
 
+class ArgparseCompleter:
+    """A completer function for ArgParserAlias commands"""
+
+    def __init__(self, parser: ap.ArgumentParser, command, **kwargs):
+        args = tuple(c.value for c in command.args[: command.arg_index])
+
+        self.parser, self.remaining_args = self.get_parser(parser, args[1:])
+
+        self.long_opts_only = XSH.env.get("ALIAS_COMPLETIONS_OPTIONS_LONGEST", False)
+        self.command = command
+        kwargs["command"] = command
+        # will be sent to completer function
+        self.kwargs = kwargs
+
+    @staticmethod
+    def get_parser(parser, args) -> tp.Tuple[ap.ArgumentParser, tp.Tuple[str, ...]]:
+        """Check for sub-parsers"""
+        sub_parsers = {}
+        for act in parser._get_positional_actions():
+            if act.nargs == ap.PARSER:
+                sub_parsers = act.choices  # there should be only one subparser
+        if sub_parsers:
+            for idx, pos in enumerate(args):
+                if pos in sub_parsers:
+                    # get the correct parser
+                    return ArgparseCompleter.get_parser(
+                        sub_parsers[pos], args[idx + 1 :]
+                    )
+        # base scenario
+        return parser, args
+
+    def filled(self, act: ap.Action) -> int:
+        """Consume remaining_args for the given action"""
+        args_len = 0
+        for arg in self.remaining_args:
+            if arg and arg[0] in self.parser.prefix_chars:
+                # stop when other --option explicitly given
+                break
+            args_len += 1
+        nargs = (
+            act.nargs
+            if isinstance(act.nargs, int)
+            else args_len + 1
+            if act.nargs in {ap.ONE_OR_MORE, ap.ZERO_OR_MORE}
+            else 1
+        )
+        if len(self.remaining_args) >= nargs:
+            # consume n-number of args
+            self.remaining_args = self.remaining_args[nargs:]
+            # complete for next action
+            return True
+        return False
+
+    def _complete(self, act: ap.Action, **kwargs):
+        if hasattr(act, "completer") and callable(act.completer):  # type: ignore
+            # call the completer function
+            kwargs.update(self.kwargs)
+            yield from act.completer(xsh=XSH, action=act, completer=self, **kwargs)  # type: ignore
+
+        if (
+            hasattr(act, "choices")
+            and act.choices
+            and not isinstance(act.choices, dict)
+        ):
+            # any sequence or iterable
+            yield from act.choices
+
+    def _complete_pos(self, act):
+        # even subparserAction can have completer attribute set
+        yield from self._complete(act)
+
+        if isinstance(act.choices, dict):  # sub-parsers
+            for choice, sub_parser in act.choices.items():
+                yield RichCompletion(
+                    choice,
+                    description=sub_parser.description or "",
+                    append_space=True,
+                )
+
+    def complete(self):
+        # options will come before/after positionals
+        options = {act: None for act in self.parser._get_optional_actions()}
+
+        # remove options that are already filled
+        opt_completions = self._complete_options(options)
+        if opt_completions:
+            yield from opt_completions
+            return
+
+        for act in self.parser._get_positional_actions():
+            # number of arguments it consumes
+            if self.filled(act):
+                continue
+            yield from self._complete_pos(act)
+            # close after a valid positional arg completion
+            break
+
+        opt_completions = self._complete_options(options)
+        if opt_completions:
+            yield from opt_completions
+            return
+
+        # complete remaining options only if requested or enabled
+        show_opts = XSH.env.get("ALIAS_COMPLETIONS_OPTIONS_BY_DEFAULT", False)
+        if not show_opts:
+            if not (
+                self.command.prefix
+                and self.command.prefix[0] in self.parser.prefix_chars
+            ):
+                return
+
+        # in the end after positionals show remaining unfilled options
+        for act in options:
+            for flag in sorted(act.option_strings, key=len, reverse=True):
+                desc = ""
+                if act.help:
+                    formatter = self.parser._get_formatter()
+                    try:
+                        desc = formatter._expand_help(act)
+                    except KeyError:
+                        desc = act.help
+                yield RichCompletion(flag, description=desc)
+                if self.long_opts_only:
+                    break
+
+    def _complete_options(self, options):
+        while self.remaining_args:
+            arg = self.remaining_args[0]
+            act_res = self.parser._parse_optional(arg)
+            if not act_res:
+                # it is not a option string: pass
+                break
+            # it is a valid option and advance
+            self.remaining_args = self.remaining_args[1:]
+            act, _, value = act_res
+
+            # remove the found option
+            # todo: not remove if append/extend
+            options.pop(act, None)
+
+            if self.filled(act):
+                continue
+            # stop suggestion until current option is complete
+            return self._complete(act)
+
+
 class ArgParserAlias:
     """Provides a structure to the Alias. The parser is lazily loaded.
 
@@ -392,9 +579,17 @@ class ArgParserAlias:
 
     Examples
     ---------
-        For usage please check ``xonsh.completers.completer.py`` module.
+        For usage please check :py:mod:`xonsh.completers.completer`
     """
 
+    class Error(Exception):
+        """Special case, when raised, the traceback will not be shown.
+        Instead the process with exit with error code and message"""
+
+        def __init__(self, message: str, errno=1):
+            super().__init__(message)
+            self.errno = errno
+
     def __init__(self, threadable=True, **kwargs) -> None:
         if not threadable:
             from xonsh.tools import unthreadable
@@ -402,6 +597,8 @@ class ArgParserAlias:
             unthreadable(self)
         self._parser = None
         self.kwargs = kwargs
+        self.stdout = None
+        self.stderr = None
 
     def build(self):
         """Sub-classes should return constructed ArgumentParser"""
@@ -409,6 +606,24 @@ class ArgParserAlias:
             return self.create_parser(**self.kwargs)
         raise NotImplementedError
 
+    def hook_pre_add_argument(self, param: str, func, flags, kwargs):
+        """Hook to update arguments that are passed to parser.add_argument"""
+        return flags, kwargs
+
+    def hook_post_add_argument(
+        self,
+        *,
+        parser: "ArgParser|ap.ArgumentParser",
+        action: "ap.Action",
+        param: str,
+        func: "tp.Callable",
+    ):
+        """Hook into parser.add_argument step.
+
+        Can be used to update action's attributes
+        """
+        return
+
     @property
     def parser(self):
         if self._parser is None:
@@ -421,11 +636,30 @@ class ArgParserAlias:
         has_args = has_args or bool(allowed_params)
         if has_args:
             kwargs.setdefault("empty_help", False)
+
         parser = make_parser(func, **kwargs)
         if has_args:
             add_args(parser, func, allowed_params=allowed_params)
         return parser
 
+    def xonsh_complete(self, command, **kwargs):
+        completer = ArgparseCompleter(self.parser, command=command, **kwargs)
+        yield from completer.complete()
+
+    def write_to(self, stream: str, *args, **kwargs):
+        value = getattr(self, stream)
+        out = getattr(sys, stream) if value is None else value
+        kwargs.setdefault("file", out)
+        print(*args, **kwargs)
+
+    def err(self, *args, **kwargs):
+        """Write text to error stream"""
+        self.write_to("stderr", *args, **kwargs)
+
+    def out(self, *args, **kwargs):
+        """Write text to output stream"""
+        self.write_to("stdout", *args, **kwargs)
+
     def __call__(
         self,
         args,
@@ -436,23 +670,35 @@ class ArgParserAlias:
         stack=None,
         **kwargs,
     ):
-        return dispatch(
-            self.parser,
-            args,
-            _parser=self.parser,
-            _args=args,
-            _stdin=stdin,
-            _stdout=stdout,
-            _stderr=stderr,
-            _spec=spec,
-            _stack=stack,
-            **kwargs,
-        )
+        self.stdout = stdout
+        self.stderr = stderr
+        try:
+            result = dispatch(
+                self.parser,
+                args,
+                _parser=self.parser,
+                _args=args,
+                _stdin=stdin,
+                _stdout=stdout,
+                _stderr=stderr,
+                _spec=spec,
+                _stack=stack,
+                **kwargs,
+            )
+        except self.Error as ex:
+            self.err(f"Error: {ex}")
+            sys.exit(getattr(ex, "errno", 1))
+        finally:
+            # free the reference to input/output. Otherwise it will result in errors
+            self.stdout = None
+            self.stderr = None
+        return result
 
 
 __all__ = (
     "Arg",
     "ArgParserAlias",
+    "ArgparseCompleter",
     "Annotated",
     "ArgParser",
     "make_parser",
