@@ -5,10 +5,10 @@ color codes. Parts of this file were originally forked from Micah Elliott
 http://MicahElliott.com Copyright (C) 2011 Micah Elliott. All rights reserved.
 WTFPL http://sam.zoy.org/wtfpl/
 """
-import re
 import math
+import re
 
-from xonsh.lazyasd import lazyobject, LazyObject
+from xonsh.lazyasd import LazyObject, lazyobject
 from xonsh.tools import print_warning
 
 _NO_COLOR_WARNING_SHOWN = False
@@ -18,6 +18,16 @@ RE_BACKGROUND = LazyObject(
 )
 
 
+class COLORS:
+    """constants"""
+
+    RESET = "{RESET}"
+    RED = "{RED}"
+    GREEN = "{GREEN}"
+    BOLD_RED = "{BOLD_RED}"
+    BOLD_GREEN = "{BOLD_GREEN}"
+
+
 @lazyobject
 def KNOWN_XONSH_COLORS():
     """These are the minimum number of colors that need to be implemented by
@@ -454,9 +464,9 @@ def RE_RGB6():
 
 def rgb_to_ints(rgb):
     if len(rgb) == 6:
-        return tuple([int(h, 16) for h in RE_RGB6.split(rgb)[1:4]])
+        return tuple(int(h, 16) for h in RE_RGB6.split(rgb)[1:4])
     else:
-        return tuple([int(h * 2, 16) for h in RE_RGB3.split(rgb)[1:4]])
+        return tuple(int(h * 2, 16) for h in RE_RGB3.split(rgb)[1:4])
 
 
 def short_to_ints(short):
@@ -464,12 +474,14 @@ def short_to_ints(short):
     return rgb_to_ints(short2rgb(short))
 
 
-def color_dist(x, y):
+def color_dist(x, y) -> float:
     return math.sqrt((x[0] - y[0]) ** 2 + (x[1] - y[1]) ** 2 + (x[2] - y[2]) ** 2)
 
 
 def find_closest_color(x, palette):
-    return min(sorted(palette.keys())[::-1], key=lambda k: color_dist(x, palette[k]))
+    return min(
+        sorted(palette.keys(), reverse=True), key=lambda k: color_dist(x, palette[k])
+    )
 
 
 def make_palette(strings):
