@@ -43,7 +43,7 @@ def _windows_bash_command(env=None):
             out = subprocess.check_output(
                 [bash_on_path, "--version"],
                 stderr=subprocess.PIPE,
-                universal_newlines=True,
+                text=True,
             )
         except subprocess.CalledProcessError:
             bash_works = False
@@ -387,7 +387,7 @@ def bash_completions(
     try:
         out = subprocess.check_output(
             [command, "-c", script],
-            universal_newlines=True,
+            text=True,
             stderr=subprocess.PIPE,
             env=env,
         )
@@ -432,6 +432,10 @@ def bash_completions(
     # to be incorrectly calculated, so it needs to be fixed here
     if "=" in prefix and "=" not in commprefix:
         strip_len = prefix.index("=") + 1
+    # Fix case where remote git branch is being deleted
+    # (e.g. 'git push origin :dev-branch')
+    elif ":" in prefix and ":" not in commprefix:
+        strip_len = prefix.index(":") + 1
 
     return out, max(len(prefix) - strip_len, 0)
 
