@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """Implements the xonsh history backend via sqlite3."""
 import collections
 import json
@@ -8,9 +7,9 @@ import sys
 import threading
 import time
 
+import xonsh.tools as xt
 from xonsh.built_ins import XSH
 from xonsh.history.base import History
-import xonsh.tools as xt
 
 XH_SQLITE_CACHE = threading.local()
 XH_SQLITE_TABLE_NAME = "xonsh_history"
@@ -90,14 +89,14 @@ ON {XH_SQLITE_TABLE_NAME}(inp);"""
 
 def _xh_sqlite_get_frequency(cursor, input):
     # type: (sqlite3.Cursor, str) -> int
-    sql = "SELECT sum(frequency) FROM {} WHERE inp=?".format(XH_SQLITE_TABLE_NAME)
+    sql = f"SELECT sum(frequency) FROM {XH_SQLITE_TABLE_NAME} WHERE inp=?"
     cursor.execute(sql, (input,))
     return cursor.fetchone()[0] or 0
 
 
 def _xh_sqlite_erase_dups(cursor, input):
     freq = _xh_sqlite_get_frequency(cursor, input)
-    sql = "DELETE FROM {} WHERE inp=?".format(XH_SQLITE_TABLE_NAME)
+    sql = f"DELETE FROM {XH_SQLITE_TABLE_NAME} WHERE inp=?"
     cursor.execute(sql, (input,))
     return freq
 
@@ -283,7 +282,7 @@ class SqliteHistory(History):
         setattr(XH_SQLITE_CACHE, XH_SQLITE_CREATED_SQL_TBL, False)
 
     def append(self, cmd):
-        if not self.remember_history:
+        if (not self.remember_history) or self.is_ignored(cmd):
             return
         envs = XSH.env
         inp = cmd["inp"].rstrip()
