@@ -1,27 +1,83 @@
-# -*- coding: utf-8 -*-
 """Job control for the xonsh shell."""
-import os
-import sys
-import time
+import collections
+import contextlib
 import ctypes
+import os
 import signal
 import subprocess
-import collections
+import sys
+import threading
+import time
 import typing as tp
 
 from xonsh.built_ins import XSH
 from xonsh.cli_utils import Annotated, Arg, ArgParserAlias
 from xonsh.completers.tools import RichCompletion
 from xonsh.lazyasd import LazyObject
-from xonsh.platform import FD_STDERR, ON_DARWIN, ON_WINDOWS, ON_CYGWIN, ON_MSYS, LIBC
-from xonsh.tools import unthreadable
+from xonsh.platform import FD_STDERR, LIBC, ON_CYGWIN, ON_DARWIN, ON_MSYS, ON_WINDOWS
+from xonsh.tools import on_main_thread, unthreadable
 
-# there is not much cost initing deque
-tasks: tp.Deque[int] = collections.deque()
 # Track time stamp of last exit command, so that two consecutive attempts to
 # exit can kill all jobs and exit.
 _last_exit_time: tp.Optional[float] = None
 
+# Thread-local data for job control. Allows threadable callable aliases
+# (ProcProxyThread) to maintain job control information separate from the main
+# thread.
+#
+# _jobs_thread_local.tasks is a queue used to keep track of the tasks. On
+# the main thread, it is set to _tasks_main.
+#
+# _jobs_thread_local.jobs is a dictionary used to keep track of the jobs.
+# On the main thread, it is set to XSH.all_jobs.
+_jobs_thread_local = threading.local()
+
+# Task queue for the main thread
+# The use_main_jobs context manager uses this variable to access the tasks on
+# the main thread.
+_tasks_main: tp.Deque[int] = collections.deque()
+
+
+@contextlib.contextmanager
+def use_main_jobs():
+    """Context manager that replaces a thread's task queue and job dictionary
+    with those of the main thread
+
+    This allows another thread (e.g. the commands jobs, disown, and bg) to
+    handle the main thread's job control.
+    """
+    old_tasks = get_tasks()
+    old_jobs = get_jobs()
+    try:
+        _jobs_thread_local.tasks = _tasks_main
+        _jobs_thread_local.jobs = XSH.all_jobs
+        yield
+    finally:
+        _jobs_thread_local.tasks = old_tasks
+        _jobs_thread_local.jobs = old_jobs
+
+
+def get_tasks() -> tp.Deque[int]:
+    try:
+        return _jobs_thread_local.tasks
+    except AttributeError:
+        if on_main_thread():
+            _jobs_thread_local.tasks = _tasks_main
+        else:
+            _jobs_thread_local.tasks = collections.deque()
+        return _jobs_thread_local.tasks
+
+
+def get_jobs() -> tp.Dict[int, tp.Dict]:
+    try:
+        return _jobs_thread_local.jobs
+    except AttributeError:
+        if on_main_thread():
+            _jobs_thread_local.jobs = XSH.all_jobs
+        else:
+            _jobs_thread_local.jobs = {}
+        return _jobs_thread_local.jobs
+
 
 if ON_DARWIN:
 
@@ -37,7 +93,6 @@ if ON_DARWIN:
             except ProcessLookupError:
                 pass
 
-
 elif ON_WINDOWS:
     pass
 elif ON_CYGWIN or ON_MSYS:
@@ -53,7 +108,6 @@ elif ON_CYGWIN or ON_MSYS:
                 except Exception:
                     pass
 
-
 else:
 
     def _send_signal(job, signal):
@@ -79,6 +133,8 @@ if ON_WINDOWS:
             stderr=subprocess.STDOUT,
         )
 
+    _hup = _kill  # there is no equivalent of SIGHUP on Windows
+
     def ignore_sigtstp():
         pass
 
@@ -108,7 +164,6 @@ if ON_WINDOWS:
                     pass  # ignore error if process closed before we got here
         return wait_for_active_job(last_task=active_task)
 
-
 else:
 
     def _continue(job):
@@ -118,6 +173,9 @@ else:
     def _kill(job):
         _send_signal(job, signal.SIGKILL)
 
+    def _hup(job):
+        _send_signal(job, signal.SIGHUP)
+
     def ignore_sigtstp():
         signal.signal(signal.SIGTSTP, signal.SIG_IGN)
 
@@ -245,6 +303,7 @@ def _safe_wait_for_active_job(last_task=None, backgrounded=False):
 
 def get_next_task():
     """Get the next active task and put it on top of the queue"""
+    tasks = get_tasks()
     _clear_dead_jobs()
     selected_task = None
     for tid in tasks:
@@ -260,31 +319,33 @@ def get_next_task():
 
 
 def get_task(tid):
-    return XSH.all_jobs[tid]
+    return get_jobs()[tid]
 
 
 def _clear_dead_jobs():
     to_remove = set()
+    tasks = get_tasks()
     for tid in tasks:
         obj = get_task(tid)["obj"]
         if obj is None or obj.poll() is not None:
             to_remove.add(tid)
     for job in to_remove:
         tasks.remove(job)
-        del XSH.all_jobs[job]
+        del get_jobs()[job]
 
 
 def format_job_string(num: int) -> str:
     try:
-        job = XSH.all_jobs[num]
+        job = get_jobs()[num]
     except KeyError:
         return ""
+    tasks = get_tasks()
     pos = "+" if tasks[0] == num else "-" if tasks[1] == num else " "
     status = job["status"]
     cmd = " ".join([" ".join(i) if isinstance(i, list) else i for i in job["cmds"]])
     pid = job["pids"][-1]
     bg = " &" if job["bg"] else ""
-    return "[{}]{} {}: {}{} ({})".format(num, pos, status, cmd, bg, pid)
+    return f"[{num}]{pos} {status}: {cmd}{bg} ({pid})"
 
 
 def print_one_job(num, outfile=sys.stdout):
@@ -298,7 +359,7 @@ def get_next_job_number():
     """Get the lowest available unique job number (for the next job created)."""
     _clear_dead_jobs()
     i = 1
-    while i in XSH.all_jobs:
+    while i in get_jobs():
         i += 1
     return i
 
@@ -308,8 +369,8 @@ def add_job(info):
     num = get_next_job_number()
     info["started"] = time.time()
     info["status"] = "running"
-    tasks.appendleft(num)
-    XSH.all_jobs[num] = info
+    get_tasks().appendleft(num)
+    get_jobs()[num] = info
     if info["bg"] and XSH.env.get("XONSH_INTERACTIVE"):
         print_one_job(num)
 
@@ -317,7 +378,7 @@ def add_job(info):
 def clean_jobs():
     """Clean up jobs for exiting shell
 
-    In non-interactive mode, kill all jobs.
+    In non-interactive mode, send SIGHUP to all jobs.
 
     In interactive mode, check for suspended or background jobs, print a
     warning if any exist, and return False. Otherwise, return True.
@@ -326,7 +387,7 @@ def clean_jobs():
     if XSH.env["XONSH_INTERACTIVE"]:
         _clear_dead_jobs()
 
-        if XSH.all_jobs:
+        if get_jobs():
             global _last_exit_time
             hist = XSH.history
             if hist is not None and len(hist.tss) > 0:
@@ -339,9 +400,9 @@ def clean_jobs():
                 # part of the last command and is now being called again
                 # immediately. Kill jobs and exit without reminder about
                 # unfinished jobs in this case.
-                kill_all_jobs()
+                hup_all_jobs()
             else:
-                if len(XSH.all_jobs) > 1:
+                if len(get_jobs()) > 1:
                     msg = "there are unfinished jobs"
                 else:
                     msg = "there is an unfinished job"
@@ -350,7 +411,7 @@ def clean_jobs():
                     # The Ctrl+D binding for prompt_toolkit already inserts a
                     # newline
                     print()
-                print("xonsh: {}".format(msg), file=sys.stderr)
+                print(f"xonsh: {msg}", file=sys.stderr)
                 print("-" * 5, file=sys.stderr)
                 jobs([], stdout=sys.stderr)
                 print("-" * 5, file=sys.stderr)
@@ -361,20 +422,21 @@ def clean_jobs():
                 jobs_clean = False
                 _last_exit_time = time.time()
     else:
-        kill_all_jobs()
+        hup_all_jobs()
 
     return jobs_clean
 
 
-def kill_all_jobs():
+def hup_all_jobs():
     """
-    Send SIGKILL to all child processes (called when exiting xonsh).
+    Send SIGHUP to all child processes (called when exiting xonsh).
     """
     _clear_dead_jobs()
-    for job in XSH.all_jobs.values():
-        _kill(job)
+    for job in get_jobs().values():
+        _hup(job)
 
 
+@use_main_jobs()
 def jobs(args, stdin=None, stdout=sys.stdout, stderr=None):
     """
     xonsh command: jobs
@@ -382,16 +444,17 @@ def jobs(args, stdin=None, stdout=sys.stdout, stderr=None):
     Display a list of all current jobs.
     """
     _clear_dead_jobs()
-    for j in tasks:
+    for j in get_tasks():
         print_one_job(j, outfile=stdout)
     return None, None
 
 
-def resume_job(args, wording):
+def resume_job(args, wording: tp.Literal["fg", "bg"]):
     """
     used by fg and bg to resume a job either in the foreground or in the background.
     """
     _clear_dead_jobs()
+    tasks = get_tasks()
     if len(tasks) == 0:
         return "", "There are currently no suspended jobs"
 
@@ -406,12 +469,12 @@ def resume_job(args, wording):
             else:
                 tid = int(args[0])
         except (ValueError, IndexError):
-            return "", "Invalid job: {}\n".format(args[0])
+            return "", f"Invalid job: {args[0]}\n"
 
-        if tid not in XSH.all_jobs:
-            return "", "Invalid job: {}\n".format(args[0])
+        if tid not in get_jobs():
+            return "", f"Invalid job: {args[0]}\n"
     else:
-        return "", "{} expects 0 or 1 arguments, not {}\n".format(wording, len(args))
+        return "", f"{wording} expects 0 or 1 arguments, not {len(args)}\n"
 
     # Put this one on top of the queue
     tasks.remove(tid)
@@ -423,7 +486,9 @@ def resume_job(args, wording):
     if XSH.env.get("XONSH_INTERACTIVE"):
         print_one_job(tid)
     pipeline = job["pipeline"]
-    pipeline.resume(job)
+    pipeline.resume(
+        job, tee_output=(wording == "fg")
+    )  # do not tee output for background jobs
 
 
 @unthreadable
@@ -438,6 +503,7 @@ def fg(args, stdin=None):
     return resume_job(args, wording="fg")
 
 
+@use_main_jobs()
 def bg(args, stdin=None):
     """xonsh command: bg
 
@@ -446,7 +512,7 @@ def bg(args, stdin=None):
     """
     res = resume_job(args, wording="bg")
     if res is None:
-        curtask = get_task(tasks[0])
+        curtask = get_task(get_tasks()[0])
         curtask["bg"] = True
         _continue(curtask)
     else:
@@ -455,10 +521,11 @@ def bg(args, stdin=None):
 
 def job_id_completer(xsh, **_):
     """Return currently running jobs ids"""
-    for job_id in xsh.all_jobs:
+    for job_id in get_jobs():
         yield RichCompletion(str(job_id), description=format_job_string(job_id))
 
 
+@use_main_jobs()
 def disown_fn(
     job_ids: Annotated[
         tp.Sequence[int], Arg(type=int, nargs="*", completer=job_id_completer)
@@ -481,6 +548,7 @@ def disown_fn(
         Automatically continue stopped jobs when they are disowned, equivalent to setting $AUTO_CONTINUE=True
     """
 
+    tasks = get_tasks()
     if len(tasks) == 0:
         return "", "There are no active jobs"
 
@@ -504,7 +572,7 @@ def disown_fn(
 
         # Stop tracking this task
         tasks.remove(tid)
-        del XSH.all_jobs[tid]
+        del get_jobs()[tid]
         messages.append(f"Removed job {tid} ({current_task['status']})")
 
     if messages:
