@@ -1,34 +1,31 @@
-# -*- coding: utf-8 -*-
 """The main xonsh script."""
-import os
-import sys
-import enum
 import argparse
 import builtins
 import contextlib
+import enum
+import os
 import signal
+import sys
 import traceback
 
-from xonsh import __version__
-from xonsh.timings import setup_timings
-from xonsh.lazyasd import lazyobject
-from xonsh.shell import Shell
-from xonsh.pretty import pretty
-from xonsh.execer import Execer
-from xonsh.jobs import ignore_sigtstp
-from xonsh.tools import print_color, to_bool_or_int
-from xonsh.platform import HAS_PYGMENTS, ON_WINDOWS
-from xonsh.codecache import run_script_with_cache, run_code_with_cache
-from xonsh.xonfig import print_welcome_screen
-from xonsh.xontribs import xontribs_load
-from xonsh.lazyimps import pygments, pyghooks
-from xonsh.imphooks import install_import_hooks
-from xonsh.events import events
-from xonsh.environ import xonshrc_context, make_args_env
-from xonsh.built_ins import XSH
-
 import xonsh.procs.pipelines as xpp
-
+from xonsh import __version__
+from xonsh.built_ins import XSH
+from xonsh.codecache import run_code_with_cache, run_script_with_cache
+from xonsh.environ import make_args_env, xonshrc_context
+from xonsh.events import events
+from xonsh.execer import Execer
+from xonsh.imphooks import install_import_hooks
+from xonsh.jobs import ignore_sigtstp
+from xonsh.lazyasd import lazyobject
+from xonsh.lazyimps import pyghooks, pygments
+from xonsh.platform import HAS_PYGMENTS, ON_WINDOWS
+from xonsh.pretty import pretty
+from xonsh.shell import Shell
+from xonsh.timings import setup_timings
+from xonsh.tools import display_error_message, print_color, to_bool_or_int
+from xonsh.xonfig import print_welcome_screen
+from xonsh.xontribs import auto_load_xontribs_from_entrypoints, xontribs_load
 
 events.transmogrify("on_post_init", "LoadEvent")
 events.doc(
@@ -77,6 +74,16 @@ NOTE: All the caveats of the ``atexit`` module also apply to this event.
 """,
 )
 
+events.transmogrify("on_xontribs_loaded", "LoadEvent")
+events.doc(
+    "on_xontribs_loaded",
+    """
+on_xontribs_loaded() -> None
+
+Fired after external xontribs with ``entrypoints defined`` are loaded.
+""",
+)
+
 events.transmogrify("on_pre_rc", "LoadEvent")
 events.doc(
     "on_pre_rc",
@@ -115,7 +122,7 @@ def path_argument(s):
 
     s = os.path.abspath(os.path.expanduser(s))
     if not os.path.exists(s):
-        msg = "{0!r} must be a valid path to a file or directory".format(s)
+        msg = f"{s!r} must be a valid path to a file or directory"
         raise argparse.ArgumentTypeError(msg)
     return s
 
@@ -267,33 +274,10 @@ class XonshMode(enum.Enum):
     interactive = 3
 
 
-def start_services(shell_kwargs, args, pre_env=None):
-    """Starts up the essential services in the proper order.
-    This returns the environment instance as a convenience.
-    """
-    if pre_env is None:
-        pre_env = {}
-    install_import_hooks()
-    # create execer, which loads builtins
-    ctx = shell_kwargs.get("ctx", {})
-    debug = to_bool_or_int(os.getenv("XONSH_DEBUG", "0"))
-    events.on_timingprobe.fire(name="pre_execer_init")
-    execer = Execer(
-        xonsh_ctx=ctx,
-        debug_level=debug,
-        scriptcache=shell_kwargs.get("scriptcache", True),
-        cacheall=shell_kwargs.get("cacheall", False),
-    )
-    events.on_timingprobe.fire(name="post_execer_init")
-    # load rc files
-    login = shell_kwargs.get("login", True)
-    rc_cli = shell_kwargs.get("rc")
-    env = XSH.env
-    for k, v in pre_env.items():
-        env[k] = v
-
+def _get_rc_files(shell_kwargs: dict, args, env):
     # determine which RC files to load, including whether any RC directories
     # should be scanned for such files
+    rc_cli = shell_kwargs.get("rc")
     if shell_kwargs.get("norc") or (
         args.mode != XonshMode.interactive
         and not args.force_interactive
@@ -301,23 +285,72 @@ def start_services(shell_kwargs, args, pre_env=None):
     ):
         # if --no-rc was passed, or we're not in an interactive shell and
         # interactive mode was not forced, then disable loading RC files and dirs
-        rc = ()
-        rcd = ()
-    elif rc_cli:
+        return (), ()
+
+    if rc_cli:
         # if an explicit --rc was passed, then we should load only that RC
         # file, and nothing else (ignore both XONSHRC and XONSHRC_DIR)
-        rc = [r for r in rc_cli if os.path.isfile(r)]
-        rcd = [r for r in rc_cli if os.path.isdir(r)]
-    else:
-        # otherwise, get the RC files from XONSHRC, and RC dirs from XONSHRC_DIR
-        rc = env.get("XONSHRC")
-        rcd = env.get("XONSHRC_DIR")
+        rc = tuple(r for r in rc_cli if os.path.isfile(r))
+        rcd = tuple(r for r in rc_cli if os.path.isdir(r))
+        return rc, rcd
 
+    # otherwise, get the RC files from XONSHRC, and RC dirs from XONSHRC_DIR
+    rc = env.get("XONSHRC")
+    rcd = env.get("XONSHRC_DIR")
+    return rc, rcd
+
+
+def _load_rc_files(shell_kwargs: dict, args, env, execer, ctx):
     events.on_pre_rc.fire()
+    # load rc files
+    login = shell_kwargs.get("login", True)
+    rc, rcd = _get_rc_files(shell_kwargs, args, env)
     XSH.rc_files = xonshrc_context(
         rcfiles=rc, rcdirs=rcd, execer=execer, ctx=ctx, env=env, login=login
     )
     events.on_post_rc.fire()
+
+
+def _autoload_xontribs(env):
+    events.on_timingprobe.fire(name="pre_xontribs_autoload")
+    disabled = env.get("XONTRIBS_AUTOLOAD_DISABLED", False)
+    if disabled is True:
+        return
+    blocked_xontribs = disabled or ()
+    auto_load_xontribs_from_entrypoints(
+        blocked_xontribs, verbose=bool(env.get("XONSH_DEBUG", False))
+    )
+    events.on_xontribs_loaded.fire()
+    events.on_timingprobe.fire(name="post_xontribs_autoload")
+
+
+def start_services(shell_kwargs, args, pre_env=None):
+    """Starts up the essential services in the proper order.
+    This returns the environment instance as a convenience.
+    """
+    if pre_env is None:
+        pre_env = {}
+    # create execer, which loads builtins
+    ctx = shell_kwargs.get("ctx", {})
+    debug = to_bool_or_int(os.getenv("XONSH_DEBUG", "0"))
+    events.on_timingprobe.fire(name="pre_execer_init")
+    execer = Execer(
+        filename="<stdin>",
+        debug_level=debug,
+        scriptcache=shell_kwargs.get("scriptcache", True),
+        cacheall=shell_kwargs.get("cacheall", False),
+    )
+    XSH.load(ctx=ctx, execer=execer)
+    events.on_timingprobe.fire(name="post_execer_init")
+
+    install_import_hooks(execer)
+
+    env = XSH.env
+    for k, v in pre_env.items():
+        env[k] = v
+
+    _autoload_xontribs(env)
+    _load_rc_files(shell_kwargs, args, env, execer, ctx)
     # create shell
     XSH.shell = Shell(execer=execer, **shell_kwargs)
     ctx["__name__"] = "__main__"
@@ -422,7 +455,7 @@ def _failback_to_other_shells(args, err):
     if foreign_shell:
         traceback.print_exc()
         print("Xonsh encountered an issue during launch", file=sys.stderr)
-        print("Failback to {}".format(foreign_shell), file=sys.stderr)
+        print(f"Failback to {foreign_shell}", file=sys.stderr)
         os.execlp(foreign_shell, foreign_shell)
     else:
         raise err
@@ -456,6 +489,10 @@ def main_xonsh(args):
     if shell and not env["XONSH_INTERACTIVE"]:
         shell.ctx.update({"exit": sys.exit})
 
+    # store a sys.exc_info() tuple to record any exception that might occur in the user code that we are about to execute
+    # if this does not change, no exceptions were thrown. Otherwise, print a traceback that does not expose xonsh internals
+    exc_info = None, None, None
+
     try:
         if args.mode == XonshMode.interactive:
             # enter the shell
@@ -464,8 +501,10 @@ def main_xonsh(args):
             env["XONSH_INTERACTIVE"] = True
 
             ignore_sigtstp()
-            if env["XONSH_INTERACTIVE"] and not any(
-                os.path.isfile(i) for i in env["XONSHRC"]
+            if (
+                env["XONSH_INTERACTIVE"]
+                and not any(os.path.isfile(i) for i in env["XONSHRC"])
+                and not any(os.path.isdir(i) for i in env["XONSHRC_DIR"])
             ):
                 print_welcome_screen()
             events.on_pre_cmdloop.fire()
@@ -475,8 +514,12 @@ def main_xonsh(args):
                 events.on_post_cmdloop.fire()
         elif args.mode == XonshMode.single_command:
             # run a single command and exit
-            run_code_with_cache(
-                args.command.lstrip(), shell.execer, glb=shell.ctx, mode="single"
+            exc_info = run_code_with_cache(
+                args.command.lstrip(),
+                "<string>",
+                shell.execer,
+                glb=shell.ctx,
+                mode="single",
             )
             if history is not None and history.last_cmd_rtn is not None:
                 exit_code = history.last_cmd_rtn
@@ -488,26 +531,43 @@ def main_xonsh(args):
                 env.update(make_args_env())  # $ARGS is not sys.argv
                 env["XONSH_SOURCE"] = path
                 shell.ctx.update({"__file__": args.file, "__name__": "__main__"})
-                run_script_with_cache(
+                exc_info = run_script_with_cache(
                     args.file, shell.execer, glb=shell.ctx, loc=None, mode="exec"
                 )
             else:
-                print("xonsh: {0}: No such file or directory.".format(args.file))
+                print(f"xonsh: {args.file}: No such file or directory.")
                 exit_code = 1
         elif args.mode == XonshMode.script_from_stdin:
             # run a script given on stdin
             code = sys.stdin.read()
-            run_code_with_cache(
-                code, shell.execer, glb=shell.ctx, loc=None, mode="exec"
+            exc_info = run_code_with_cache(
+                code, "<stdin>", shell.execer, glb=shell.ctx, loc=None, mode="exec"
             )
+    except SyntaxError:
+        exit_code = 1
+        debug_level = env.get("XONSH_DEBUG", 0)
+        if debug_level == 0:
+            # print error without tracktrace
+            display_error_message(sys.exc_info())
+        else:
+            # pass error to finally clause
+            exc_info = sys.exc_info()
     finally:
+        if exc_info != (None, None, None):
+            err_type, err, _ = exc_info
+            if err_type is SystemExit:
+                raise err
+            else:
+                traceback.print_exception(*exc_info)
+                exit_code = 1
         events.on_exit.fire()
-    postmain(args)
+        postmain(args)
     return exit_code
 
 
 def postmain(args=None):
     """Teardown for main xonsh entry point, accepts parsed arguments."""
+    XSH.unload()
     XSH.shell = None
 
 
@@ -562,12 +622,11 @@ def setup(
     ctx = {} if ctx is None else ctx
     # setup xonsh ctx and execer
     if not hasattr(builtins, "__xonsh__"):
-        execer = Execer(xonsh_ctx=ctx)
-        XSH.load(
-            ctx=ctx, execer=execer, shell=Shell(execer, ctx=ctx, shell_type=shell_type)
-        )
+        execer = Execer(filename="<stdin>")
+        XSH.load(ctx=ctx, execer=execer)
+        XSH.shell = Shell(execer, ctx=ctx, shell_type=shell_type)
     XSH.env.update(env)
-    install_import_hooks()
+    install_import_hooks(XSH.execer)
     XSH.aliases.update(aliases)
     if xontribs:
         xontribs_load(xontribs)
