@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 Python advanced pretty printer.  This pretty printer is intended to
 replace the old `pprint` python module which does not allow developers
@@ -78,13 +77,13 @@ without open / close parameters.  You can also use this code::
             Portions (c) 2009 by Robert Kern.
 :license: BSD License.
 """
+import collections
+import contextlib
+import datetime
 import io
 import re
 import sys
 import types
-import datetime
-import contextlib
-import collections
 
 from xonsh.lazyasd import LazyObject, lazyobject
 
@@ -119,7 +118,7 @@ def pretty(
     """
     Pretty print the object's representation.
     """
-    if hasattr(obj, "xonsh_display"):
+    if _safe_getattr(obj, "xonsh_display"):
         return obj.xonsh_display()
 
     stream = io.StringIO()
@@ -146,7 +145,7 @@ def pretty_print(
     sys.stdout.flush()
 
 
-class _PrettyPrinterBase(object):
+class _PrettyPrinterBase:
     @contextlib.contextmanager
     def indent(self, indent):
         """with statement support for indenting/dedenting."""
@@ -421,7 +420,7 @@ class RepresentationPrinter(PrettyPrinter):
         return printer
 
 
-class Printable(object):
+class Printable:
     def output(self, stream, output_width):
         return output_width
 
@@ -469,7 +468,7 @@ class Group(Printable):
         self.want_break = False
 
 
-class GroupQueue(object):
+class GroupQueue:
     def __init__(self, *groups):
         self.queue = []
         for group in groups:
@@ -640,13 +639,6 @@ def _dict_pprinter_factory(start, end, basetype=None):
             return p.text("{...}")
         p.begin_group(1, start)
         keys = obj.keys()
-        # if dict isn't large enough to be truncated, sort keys before displaying
-        if not (p.max_seq_length and len(obj) >= p.max_seq_length):
-            try:
-                keys = sorted(keys)
-            except Exception:
-                # Sometimes the keys don't sort.
-                pass
         for idx, key in p._enumerate(keys):
             if idx:
                 p.text(",")
@@ -753,7 +745,7 @@ def _exception_pprint(obj, p, cycle):
     """Base pprint for all exceptions."""
     name = getattr(obj.__class__, "__qualname__", obj.__class__.__name__)
     if obj.__class__.__module__ not in ("exceptions", "builtins"):
-        name = "%s.%s" % (obj.__class__.__module__, name)
+        name = f"{obj.__class__.__module__}.{name}"
     step = len(name) + 1
     p.begin_group(step, name + "(")
     for idx, arg in enumerate(getattr(obj, "args", ())):
