@@ -1,20 +1,19 @@
 """Command pipeline tools."""
+import io
 import os
 import re
-import io
-import sys
-import time
 import signal
-import threading
 import subprocess
+import sys
+import threading
+import time
 
-import xonsh.lazyasd as xl
-import xonsh.tools as xt
-import xonsh.platform as xp
 import xonsh.jobs as xj
+import xonsh.lazyasd as xl
+import xonsh.platform as xp
+import xonsh.tools as xt
 from xonsh.built_ins import XSH
-
-from xonsh.procs.readers import NonBlockingFDReader, ConsoleParallelReader, safe_fdclose
+from xonsh.procs.readers import ConsoleParallelReader, NonBlockingFDReader, safe_fdclose
 
 
 @xl.lazyobject
@@ -78,14 +77,14 @@ def safe_readable(handle):
     return status
 
 
-def update_fg_process_group(pipeline_group, background):
-    if background:
-        return False
+def update_process_group(pipeline_group, background):
     if not xp.ON_POSIX:
         return False
     env = XSH.env
     if not env.get("XONSH_INTERACTIVE"):
         return False
+    if background:
+        return True
     return xj.give_terminal_to(pipeline_group)
 
 
@@ -153,6 +152,13 @@ class CommandPipeline:
 
         background = self.spec.background
         pipeline_group = None
+        if xp.ON_POSIX and not xt.on_main_thread():
+            # If we are inside a ProcProxyThread, then run commands in the same
+            # process group as xonsh. This fixes case 2 of issue #4277, where
+            # the terminal is given to a command inside the ProcProxyThread,
+            # taking the terminal away from the `less` command, causing `less`
+            # to stop.
+            pipeline_group = os.getpgid(0)
         for spec in specs:
             if self.starttime is None:
                 self.starttime = time.time()
@@ -170,7 +176,7 @@ class CommandPipeline:
                 and self.captured != "object"
             ):
                 pipeline_group = proc.pid
-                if update_fg_process_group(pipeline_group, background):
+                if update_process_group(pipeline_group, background):
                     self.term_pgid = pipeline_group
             self.procs.append(proc)
         self.proc = self.procs[-1]
@@ -751,17 +757,8 @@ class HiddenCommandPipeline(CommandPipeline):
         return ""
 
 
-def pause_call_resume(p, f, *args, **kwargs):
-    """For a process p, this will call a function f with the remaining args and
-    and kwargs. If the process cannot accept signals, the function will be called.
-
-    Parameters
-    ----------
-    p : Popen object or similar
-    f : callable
-    args : remaining arguments
-    kwargs : keyword arguments
-    """
+def resume_process(p):
+    """Sends SIGCONT to a process if possible."""
     can_send_signal = (
         hasattr(p, "send_signal")
         and xp.ON_POSIX
@@ -770,15 +767,9 @@ def pause_call_resume(p, f, *args, **kwargs):
     )
     if can_send_signal:
         try:
-            p.send_signal(signal.SIGSTOP)
+            p.send_signal(signal.SIGCONT)
         except PermissionError:
             pass
-    try:
-        f(*args, **kwargs)
-    except Exception:
-        pass
-    if can_send_signal:
-        p.send_signal(signal.SIGCONT)
 
 
 class PrevProcCloser(threading.Thread):
