@@ -8,21 +8,20 @@ Vox defines several events related to the life cycle of virtual environments:
 * ``vox_on_deactivate(env: str, path: pathlib.Path) -> None``
 * ``vox_on_delete(env: str) -> None``
 """
-import os
-import sys
-import shutil
-import logging
 import collections.abc
+import logging
+import os
+import shutil
 import subprocess as sp
+import sys
+import typing
 
 from xonsh.built_ins import XSH
-from xonsh.platform import ON_POSIX, ON_WINDOWS
-
 
 # This is because builtins aren't globally created during testing.
 # FIXME: Is there a better way?
 from xonsh.events import events
-
+from xonsh.platform import ON_POSIX, ON_WINDOWS
 
 events.doc(
     "vox_on_create",
@@ -61,9 +60,11 @@ Fired after an environment is deleted (through vox).
 )
 
 
-VirtualEnvironment = collections.namedtuple(
-    "VirtualEnvironment", ["env", "bin", "lib", "inc"]
-)
+class VirtualEnvironment(typing.NamedTuple):
+    env: str
+    bin: str
+    lib: str
+    inc: str
 
 
 def _subdir_names():
@@ -121,13 +122,15 @@ class Vox(collections.abc.Mapping):
     2. ``bin``: The full path to the bin/Scripts directory of the environment
     """
 
-    def __init__(self):
+    def __init__(self, force_removals=False):
         if not XSH.env.get("VIRTUALENV_HOME"):
             home_path = os.path.expanduser("~")
             self.venvdir = os.path.join(home_path, ".virtualenvs")
             XSH.env["VIRTUALENV_HOME"] = self.venvdir
         else:
             self.venvdir = XSH.env["VIRTUALENV_HOME"]
+        self.force_removals = force_removals
+        self.sub_dirs = _subdir_names()
 
     def create(
         self,
@@ -136,6 +139,7 @@ class Vox(collections.abc.Mapping):
         system_site_packages=False,
         symlinks=False,
         with_pip=True,
+        prompt=None,
     ):
         """Create a virtual environment in $VIRTUALENV_HOME with python3's ``venv``.
 
@@ -154,10 +158,13 @@ class Vox(collections.abc.Mapping):
             environment.
         with_pip : bool
             If True, ensure pip is installed in the virtual environment. (Default is True)
+        prompt: str
+            Provides an alternative prompt prefix for this environment.
         """
-        interpreter = (
-            _get_vox_default_interpreter() if interpreter is None else interpreter
-        )
+        if interpreter is None:
+            interpreter = _get_vox_default_interpreter()
+            print(f"Using Interpreter: {interpreter}")
+
         # NOTE: clear=True is the same as delete then create.
         # NOTE: upgrade=True is its own method
         if isinstance(name, os.PathLike):
@@ -167,12 +174,18 @@ class Vox(collections.abc.Mapping):
         if not self._check_reserved(env_path):
             raise ValueError(
                 "venv can't contain reserved names ({})".format(
-                    ", ".join(_subdir_names())
+                    ", ".join(self.sub_dirs)
                 )
             )
 
-        # todo: add function to toggle system-site-packages like in pew and others
-        self._create(env_path, interpreter, system_site_packages, symlinks, with_pip)
+        self._create(
+            env_path,
+            interpreter,
+            system_site_packages,
+            symlinks,
+            with_pip,
+            prompt=prompt,
+        )
         events.vox_on_create.fire(name=name)
 
     def upgrade(self, name, symlinks=False, with_pip=True, interpreter=None):
@@ -193,13 +206,15 @@ class Vox(collections.abc.Mapping):
         with_pip : bool
             If True, ensure pip is installed in the virtual environment.
         """
-        interpreter = (
-            _get_vox_default_interpreter() if interpreter is None else interpreter
-        )
+
+        if interpreter is None:
+            interpreter = _get_vox_default_interpreter()
+            print(f"Using Interpreter: {interpreter}")
+
         # venv doesn't reload this, so we have to do it ourselves.
         # Is there a bug for this in Python? There should be.
-        env_path, bin_path = self[name]
-        cfgfile = os.path.join(env_path, "pyvenv.cfg")
+        venv = self[name]
+        cfgfile = os.path.join(venv.env, "pyvenv.cfg")
         cfgops = {}
         with open(cfgfile) as cfgfile:
             for line in cfgfile:
@@ -213,10 +228,14 @@ class Vox(collections.abc.Mapping):
             "symlinks": symlinks,
             "with_pip": with_pip,
         }
+        prompt = cfgops.get("prompt")
+        if prompt:
+            flags["prompt"] = prompt.lstrip("'\"").rstrip("'\"")
         # END things we shouldn't be doing.
 
         # Ok, do what we came here to do.
-        self._create(env_path, interpreter, upgrade=True, **flags)
+        self._create(venv.env, interpreter, upgrade=True, **flags)
+        return venv
 
     @staticmethod
     def _create(
@@ -226,9 +245,10 @@ class Vox(collections.abc.Mapping):
         symlinks=False,
         with_pip=True,
         upgrade=False,
+        prompt=None,
     ):
         version_output = sp.check_output(
-            [interpreter, "--version"], stderr=sp.STDOUT, universal_newlines=True
+            [interpreter, "--version"], stderr=sp.STDOUT, text=True
         )
 
         interpreter_major_version = int(version_output.split()[-1].split(".")[0])
@@ -248,22 +268,20 @@ class Vox(collections.abc.Mapping):
             with_pip,
             upgrade,
         ]
-        cmd = [arg for arg in cmd if arg]  # remove empty args
+        if prompt and module == "venv":
+            cmd.extend(["--prompt", prompt])
 
+        cmd = [arg for arg in cmd if arg]  # remove empty args
         logging.debug(cmd)
 
-        return_code = sp.call(cmd)
+        sp.check_call(cmd)
 
-        if return_code != 0:
-            raise SystemExit(return_code)
-
-    @staticmethod
-    def _check_reserved(name):
+    def _check_reserved(self, name):
         return (
-            os.path.basename(name) not in _subdir_names()
+            os.path.basename(name) not in self.sub_dirs
         )  # FIXME: Check the middle components, too
 
-    def __getitem__(self, name):
+    def __getitem__(self, name) -> "VirtualEnvironment":
         """Get information about a virtual environment.
 
         Parameters
@@ -307,13 +325,17 @@ class Vox(collections.abc.Mapping):
         else:
             return True
 
+    def get_binary_path(self, binary: str, *dirs: str):
+        bin_, _, _ = self.sub_dirs
+        python_exec = binary
+        if ON_WINDOWS and not python_exec.endswith(".exe"):
+            python_exec += ".exe"
+        return os.path.join(*dirs, bin_, python_exec)
+
     def __iter__(self):
         """List available virtual environments found in $VIRTUALENV_HOME."""
-        bin_, _, _ = _subdir_names()
         for dirpath, dirnames, _ in os.walk(self.venvdir):
-            python_exec = os.path.join(dirpath, bin_, "python")
-            if ON_WINDOWS:
-                python_exec += ".exe"
+            python_exec = self.get_binary_path("python", dirpath)
             if os.access(python_exec, os.X_OK):
                 yield dirpath[len(self.venvdir) + 1 :]  # +1 is to remove the separator
                 dirnames.clear()
@@ -404,6 +426,15 @@ class Vox(collections.abc.Mapping):
         except KeyError:
             # No current venv, ... fails
             pass
+
+        env_path = os.path.abspath(env_path)
+        if not self.force_removals:
+            print(f"The directory {env_path}")
+            print("and all of its content will be deleted.")
+            answer = input("Do you want to continue? [Y/n]")
+            if "n" in answer:
+                return
+
         shutil.rmtree(env_path)
 
         events.vox_on_delete.fire(name=name)
@@ -411,4 +442,9 @@ class Vox(collections.abc.Mapping):
 
 def _get_vox_default_interpreter():
     """Return the interpreter set by the $VOX_DEFAULT_INTERPRETER if set else sys.executable"""
-    return XSH.env.get("VOX_DEFAULT_INTERPRETER", sys.executable)
+    default = "python3"
+    if default in XSH.commands_cache:
+        default = XSH.commands_cache.locate_binary(default)
+    else:
+        default = sys.executable
+    return XSH.env.get("VOX_DEFAULT_INTERPRETER", default)
